//////////////////////////////////////////////////////////////////
//                                                             
// Copyright (C) 2010 Pulse Logic                       
//                    info@pulselogic.com.pl                   
//                                                             
// This source file may be used and distributed without        
// restriction provided that this copyright statement is not   
// removed from the file and that any derivative work contains 
// the original copyright notice and the associated disclaimer.
//                                                             
//     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
// POSSIBILITY OF SUCH DAMAGE.                                 
//                                                             
//////////////////////////////////////////////////////////////////
//
// Title       : AHBArbiter
// Design      : AHB Testbench
// Company     : Pulse Logic
//
//////////////////////////////////////////////////////////////////
//
// File        : AHBArbiter.v
// File Type   : RTL (Obfuscated code)
// Tool        : Xilinx WebPack
//
//////////////////////////////////////////////////////////////////
//
//Description: AHB Arbiter. The following is not supported:
//    1. Split transfers
//    2. Retry transfers
//    3. Burst termination
//////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module AHBArbiter 
    #(
    parameter P_MASTERS_NUM = 2,                         //Number of Masters
    parameter P_SLAVES_NUM = 2                           //Number of Slaves excluding dummy Slave.
    )
    (  
    input wire HCLK,                                     //Clock
    input wire HRESETn,                                  //Reset
    input wire [1 : 0] HTRANS,                           //AHB HTRANS signal
    input wire [2 : 0] HBURST,                           //AHB HBURST signal
    input wire [31 : 0] HADDR,                           //AHB HADDR signal
    input wire [P_MASTERS_NUM-1 : 0] HREADY_MST,         //Vector of HREADY signals for each master. It is taken from AHBSlaveToMaster multiplexer output.
    input wire [P_SLAVES_NUM : 0] HREADY_SLV,            //Vector of HREADY signals from each slave. 
    output wire HMASTER_VALID,                           //High state indicates that HMASTER is valid.
    input wire [P_MASTERS_NUM - 1 : 0] HBUSREQ,          //AHB HBUSREQ signal
    input wire [P_SLAVES_NUM-1 : 0] HSEL,                //AHB HSEL signal
    input wire [P_MASTERS_NUM - 1 : 0] HLOCK,            //AHB HLOCK signal
    output wire [P_MASTERS_NUM - 1 : 0] HGRANT,          //AHB HGRANT signal
    output wire [clog2(P_MASTERS_NUM-1)-1 : 0] HMASTER   //AHB HMASTER signal
    );

localparam ID_S_2b08f1c5_54955117_E = 2'b00; localparam ID_S_1d7ead3b_19138270_E = 2'b10; localparam ID_S_cf0ec30_2e07c95e_E =  2'b11; localparam ID_S_2b05646a_36485dfd_E = 2'b01;  localparam ID_S_28bbb329_3d88d889_E = 3'b000; localparam ID_S_2b091b33_54bd742c_E = 3'b001; localparam ID_S_d2c0195_4ee7bce8_E = 3'b010; localparam ID_S_c2c81c7_7090cc9d_E = 3'b011; localparam ID_S_d2c0199_4ee7bce4_E = 3'b100; localparam ID_S_c2c81cb_7090cc91_E = 3'b101; localparam ID_S_32ac3408_728816c3_E = 3'b110; localparam ID_S_11bcba7a_a8f7535_E = 3'b111;  localparam ID_S_bab60d1_5cb64e54_E = 3'b000;  localparam ID_S_1177a40_13deebe6_E = 3'b001;  localparam ID_S_1177a7e_13defbe0_E = 3'b010; localparam ID_S_1177ae3_13ded3e3_E = 3'b011;  localparam ID_S_2406c1f4_7ed0c7a6_E = 3'b100;  localparam ID_S_2406c696_7edcdeac_E = 3'b101;  localparam ID_S_2406d2d1_7ec0e02b_E = 3'b110;  localparam ID_S_24def760_78eccf50_E = 3'b111;   localparam ID_S_2b0c587b_15198a1a_E = 2'b00; localparam ID_S_be690d1_b1c9322_E = 2'b01;  localparam ID_S_ccab75d_1a31786_E = 2'b10; localparam ID_S_ce2b4d3_2b954867_E = 2'b11;   localparam ID_S_2b0de403_328a5972_E = 1'b0; localparam ID_S_d2c2432_4ec7f294_E = 1'b1;    localparam ID_S_59dd33e1_20a57556_E = 0;   function integer ID_S_609e2258_1c7667c_E (input [2 : 0] ID_S_f2e7e15_72f12829_E);    integer ID_S_b88a54f_7e3417e5_E; begin   case (ID_S_f2e7e15_72f12829_E)   ID_S_28bbb329_3d88d889_E , ID_S_2b091b33_54bd742c_E : ID_S_b88a54f_7e3417e5_E = 1;   ID_S_d2c0195_4ee7bce8_E  , ID_S_c2c81c7_7090cc9d_E :   ID_S_b88a54f_7e3417e5_E = 4;   ID_S_d2c0199_4ee7bce4_E  , ID_S_c2c81cb_7090cc91_E :   ID_S_b88a54f_7e3417e5_E = 8;   ID_S_32ac3408_728816c3_E , ID_S_11bcba7a_a8f7535_E :  ID_S_b88a54f_7e3417e5_E = 16;  endcase  ID_S_609e2258_1c7667c_E = ID_S_b88a54f_7e3417e5_E; end endfunction    function integer clog2 (input integer ID_S_1020db36_1f410d8f_E);    integer ID_S_7c9a2e19_19de53b5_E; begin    for(ID_S_7c9a2e19_19de53b5_E = 0; ID_S_1020db36_1f410d8f_E > 0; ID_S_7c9a2e19_19de53b5_E = ID_S_7c9a2e19_19de53b5_E + 1)  begin   ID_S_1020db36_1f410d8f_E = ID_S_1020db36_1f410d8f_E >> 1;  end  clog2 = ID_S_7c9a2e19_19de53b5_E; end endfunction   function integer ID_S_6bb6952c_294009fe_E (input [P_MASTERS_NUM - 1 : 0] ID_S_b887c10_7e5cd5f0_E);    integer ID_S_b88a54f_7e3417e5_E, ID_S_7c989e39_38da28fa_E; begin   for (ID_S_7c989e39_38da28fa_E = 0; ID_S_7c989e39_38da28fa_E < P_MASTERS_NUM; ID_S_7c989e39_38da28fa_E = ID_S_7c989e39_38da28fa_E + 1)  begin   if(ID_S_b887c10_7e5cd5f0_E[ID_S_7c989e39_38da28fa_E])    ID_S_b88a54f_7e3417e5_E = ID_S_7c989e39_38da28fa_E;  end   ID_S_6bb6952c_294009fe_E = ID_S_b88a54f_7e3417e5_E; end endfunction   reg ID_S_426f16ec_1589dd2e_E;  reg ID_S_3f06690e_7d948a5b_E;   integer ID_S_77937268_3e0c17f4_E;   wire ID_S_7b3773e4_15e0908a_E;  wire ID_S_5d0ed86d_1139642e_E;  integer ID_S_16edda36_47856058_E;  wire [clog2(P_MASTERS_NUM-1)-1 : 0] ID_S_7c96d2ac_75311189_E; reg ID_S_64625331_24ecad04_E; reg ID_S_6ebf36e_ff19e67_E;  wire ID_S_646d31e7_cf31997_E; wire ID_S_5f80352d_31e0ff3a_E;  wire ID_S_6c11583c_e27b482_E;   reg [P_SLAVES_NUM-1 : 0] ID_S_7e4c786e_6915dae7_E;  reg [clog2(P_MASTERS_NUM-1)-1 : 0] ID_S_6b8e2a96_180d8021_E; reg  ID_S_5a27d7c4_7f8f36ff_E;   always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   ID_S_426f16ec_1589dd2e_E <= 1'b0;  else   if (ID_S_5f80352d_31e0ff3a_E & ((HTRANS == ID_S_1d7ead3b_19138270_E) || (HTRANS == ID_S_2b08f1c5_54955117_E)) & ~ID_S_7b3773e4_15e0908a_E)     ID_S_426f16ec_1589dd2e_E <= 1'b1;   else if (ID_S_7b3773e4_15e0908a_E)    ID_S_426f16ec_1589dd2e_E <= 1'b0;   end   always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   begin    ID_S_3f06690e_7d948a5b_E <= 1'b0;   end  else    begin    ID_S_3f06690e_7d948a5b_E <= HBUSREQ[ID_S_6b8e2a96_180d8021_E];   end end   assign ID_S_5f80352d_31e0ff3a_E = HREADY_MST[ID_S_6b8e2a96_180d8021_E];   always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   ID_S_77937268_3e0c17f4_E <= 0;  else   if(ID_S_5f80352d_31e0ff3a_E == 1'b1)    begin     if((HBURST == ID_S_2b091b33_54bd742c_E) && (HTRANS != ID_S_2b08f1c5_54955117_E) && HBUSREQ[ID_S_6b8e2a96_180d8021_E])      ID_S_77937268_3e0c17f4_E <= 1;    else if (HTRANS == ID_S_2b08f1c5_54955117_E)         ID_S_77937268_3e0c17f4_E <= 1;         else if (HTRANS == ID_S_1d7ead3b_19138270_E)      ID_S_77937268_3e0c17f4_E <= (HBURST == ID_S_28bbb329_3d88d889_E) ? 1 : ID_S_609e2258_1c7667c_E(HBURST) - 1;         else if (HTRANS == ID_S_cf0ec30_2e07c95e_E)       ID_S_77937268_3e0c17f4_E <= ID_S_77937268_3e0c17f4_E - 1;    else     ID_S_77937268_3e0c17f4_E <= 0;   end  end     assign ID_S_7b3773e4_15e0908a_E = (HBURST == ID_S_2b091b33_54bd742c_E) ? (ID_S_3f06690e_7d948a5b_E & ~HBUSREQ[ID_S_6b8e2a96_180d8021_E] & ID_S_426f16ec_1589dd2e_E) :        ((ID_S_77937268_3e0c17f4_E == 1) && (HTRANS == ID_S_cf0ec30_2e07c95e_E)) || ((HTRANS == ID_S_1d7ead3b_19138270_E) && (HBURST == ID_S_28bbb329_3d88d889_E));         assign ID_S_5d0ed86d_1139642e_E = (ID_S_5f80352d_31e0ff3a_E & ID_S_7b3773e4_15e0908a_E) | (~ID_S_5a27d7c4_7f8f36ff_E);    reg [P_MASTERS_NUM - 1 : 0] ID_S_5c1fcb87_30ab14ec_E; wire [P_MASTERS_NUM - 1 : 0] ID_S_601c0d20_372eb300_E; reg [P_MASTERS_NUM - 1 : 0] ID_S_2ffc9b84_28bf5421_E; wire [P_MASTERS_NUM - 1 : 0] ID_S_2f92dcbd_78312999_E;  integer ID_S_b8891d9_7e48f46a_E, ID_S_7c989e39_38da28fa_E; reg [P_MASTERS_NUM - 1 : 0] ID_S_60112e6a_3294f3e0_E; integer ID_S_3a6d8e61_1fc9a588_E;    always @(HBUSREQ or ID_S_2ffc9b84_28bf5421_E) begin  ID_S_7c989e39_38da28fa_E = ID_S_6bb6952c_294009fe_E(ID_S_2ffc9b84_28bf5421_E);  ID_S_60112e6a_3294f3e0_E = {P_MASTERS_NUM{1'b0}};  ID_S_3a6d8e61_1fc9a588_E = 0;  ID_S_64625331_24ecad04_E = 0;    for (ID_S_b8891d9_7e48f46a_E = 0; ID_S_b8891d9_7e48f46a_E < P_MASTERS_NUM; ID_S_b8891d9_7e48f46a_E = ID_S_b8891d9_7e48f46a_E + 1)  begin    if(HBUSREQ[ID_S_7c989e39_38da28fa_E] & ~ID_S_64625331_24ecad04_E)   begin    ID_S_60112e6a_3294f3e0_E[ID_S_7c989e39_38da28fa_E] = 1'b1;      ID_S_3a6d8e61_1fc9a588_E = ID_S_7c989e39_38da28fa_E;    ID_S_64625331_24ecad04_E = 1'b1;       end      if(ID_S_7c989e39_38da28fa_E == P_MASTERS_NUM-1)    ID_S_7c989e39_38da28fa_E = 0;   else    ID_S_7c989e39_38da28fa_E = ID_S_7c989e39_38da28fa_E + 1;   end  end  always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   begin    ID_S_5c1fcb87_30ab14ec_E <= {P_MASTERS_NUM{1'b0}};    ID_S_2ffc9b84_28bf5421_E <= 1;    ID_S_16edda36_47856058_E <= 0;    ID_S_6ebf36e_ff19e67_E <= 1'b0;   end  else   if(ID_S_5d0ed86d_1139642e_E == 1'b1)    begin     if(|ID_S_60112e6a_3294f3e0_E != 1'b0)      begin             ID_S_2ffc9b84_28bf5421_E <= {ID_S_60112e6a_3294f3e0_E[P_MASTERS_NUM-2 : 0], ID_S_60112e6a_3294f3e0_E[P_MASTERS_NUM-1]};       end           ID_S_5c1fcb87_30ab14ec_E <= ID_S_60112e6a_3294f3e0_E;     ID_S_6ebf36e_ff19e67_E <= ID_S_64625331_24ecad04_E;     ID_S_16edda36_47856058_E <= ID_S_3a6d8e61_1fc9a588_E;    end end  assign ID_S_601c0d20_372eb300_E = ID_S_5d0ed86d_1139642e_E ? ID_S_60112e6a_3294f3e0_E : ID_S_5c1fcb87_30ab14ec_E; assign ID_S_646d31e7_cf31997_E = ID_S_5d0ed86d_1139642e_E ? ID_S_64625331_24ecad04_E : ID_S_6ebf36e_ff19e67_E; assign ID_S_7c96d2ac_75311189_E = ID_S_5d0ed86d_1139642e_E ? ID_S_3a6d8e61_1fc9a588_E : ID_S_16edda36_47856058_E;  reg [P_MASTERS_NUM - 1 : 0] ID_S_7f158a4f_6ef8c337_E;   always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   ID_S_7e4c786e_6915dae7_E <= {P_SLAVES_NUM{1'b0}};        else   if(ID_S_6c11583c_e27b482_E == 1'b1)    ID_S_7e4c786e_6915dae7_E <= HSEL; end    assign ID_S_6c11583c_e27b482_E = HREADY_SLV[ID_S_7e4c786e_6915dae7_E];   always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   ID_S_7f158a4f_6ef8c337_E <= {P_MASTERS_NUM{1'b0}};        else   ID_S_7f158a4f_6ef8c337_E <= ID_S_6c11583c_e27b482_E ? ID_S_601c0d20_372eb300_E : (ID_S_601c0d20_372eb300_E & ID_S_7f158a4f_6ef8c337_E);   end    assign HGRANT = ID_S_6c11583c_e27b482_E ? ID_S_601c0d20_372eb300_E : (ID_S_601c0d20_372eb300_E & ID_S_7f158a4f_6ef8c337_E);   always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   begin    ID_S_6b8e2a96_180d8021_E <= 'b0;    ID_S_5a27d7c4_7f8f36ff_E <= 1'b0;   end  else   if(ID_S_6c11583c_e27b482_E == 1'b1)    begin     ID_S_6b8e2a96_180d8021_E <= ID_S_7c96d2ac_75311189_E;     ID_S_5a27d7c4_7f8f36ff_E <= ID_S_646d31e7_cf31997_E;     end end  assign HMASTER = ID_S_6b8e2a96_180d8021_E; assign HMASTER_VALID = ID_S_5a27d7c4_7f8f36ff_E;  endmodule 