//////////////////////////////////////////////////////////////////
//                                                             
// Copyright (C) 2010 Pulse Logic                       
//                    info@pulselogic.com.pl                   
//                                                             
// This source file may be used and distributed without        
// restriction provided that this copyright statement is not   
// removed from the file and that any derivative work contains 
// the original copyright notice and the associated disclaimer.
//                                                             
//     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
// POSSIBILITY OF SUCH DAMAGE.                                 
//                                                             
//////////////////////////////////////////////////////////////////
//
// Title       : AHBBusSystem
// Design      : AHB Testbench
// Company     : Pulse Logic
//
//////////////////////////////////////////////////////////////////
//
// File        : AHBBusSystem.v
// File Type   : RTL
// Tool        : Xilinx WebPack
//
//////////////////////////////////////////////////////////////////
//
//Description: This module builds the AHB system by connecting all submodules together.
//	
//////////////////////////////////////////////////////////////////

`timescale 1ps / 1ps

module AHBBusSystem (HCLK,HRESET_N) ;

parameter P_MASTERS_NUM = 2;
parameter P_SLAVES_NUM = 2;

//Include file with parameters definition.
`include "defs.v"

input HCLK;
wire HCLK;
input HRESET_N;
wire HRESET_N;

wire HMASTER_VALID;
wire HSEL_DUMMY;
wire HWRITE_SL0;
wire HWRITE_SL1;
wire HWRITE_SL2;
wire [31:0] HADDR;
wire [(P_MASTERS_NUM * 32)-1:0] HADDR_ARR;
wire [31:0] HADDR_SL0;
wire [31:0] HADDR_SL1;
wire [31:0] HADDR_SL2;
wire [2:0] HBURST;
wire [(P_MASTERS_NUM * 3)-1:0] HBURST_ARR;
wire [2:0] HBURST_SL0;
wire [2:0] HBURST_SL1;
wire [2:0] HBURST_SL2;
wire [P_MASTERS_NUM - 1:0] HBUSREQ;
wire [P_MASTERS_NUM - 1:0] HGRANT;
wire [P_MASTERS_NUM - 1:0] HLOCK;
wire [clog2(P_MASTERS_NUM-1)-1 :0] HMASTER;
wire [(P_MASTERS_NUM * 4)-1:0] HPROT_ARR;
wire [3:0] HPROT_SL0;
wire [3:0] HPROT_SL1;
wire [3:0] HPROT_SL2;
wire [((P_SLAVES_NUM+1) * 32)-1:0] HRDATA_ARR;
wire [31:0] HRDATA_M0;
wire [31:0] HRDATA_M1;
wire [P_SLAVES_NUM:0] HREADY_ARR;
wire [P_MASTERS_NUM - 1:0] HREADY_MST;
wire [((P_SLAVES_NUM+1) * 2)-1:0] HRESP_ARR;
wire [1:0] HRESP_M0;
wire [1:0] HRESP_M1;
wire [P_SLAVES_NUM-1:0] HSEL;
wire [(P_MASTERS_NUM * 3)-1:0] HSIZE_ARR;
wire [2:0] HSIZE_SL0;
wire [2:0] HSIZE_SL1;
wire [2:0] HSIZE_SL2;
wire [1:0] HTRANS;
wire [(P_MASTERS_NUM * 2)-1:0] HTRANS_ARR;
wire [1:0] HTRANS_SL0;
wire [1:0] HTRANS_SL1;
wire [1:0] HTRANS_SL2;
wire [(P_MASTERS_NUM * 32)-1:0] HWDATA_ARR;
wire [31:0] HWDATA_SL0;
wire [31:0] HWDATA_SL1;
wire [31:0] HWDATA_SL2;
wire [P_MASTERS_NUM-1:0] HWRITE_ARR;

AHBMaster
#(
	.P_IN_FILENAME("input_1.txt"),
	.P_MASTER_NAME("0"),
	.P_OUT_FILENAME("output_1.txt"),
	.P_STOP_SIM_AT_EOF(1))
U1
(
	.HADDR(HADDR_ARR[31:0]),
	.HBURST(HBURST_ARR[2:0]),
	.HBUSREQ(HBUSREQ[0]),
	.HCLK(HCLK),
	.HGRANT(HGRANT[0]),
	.HLOCK(HLOCK[0]),
	.HPROT(HPROT_ARR[3:0]),
	.HRDATA(HRDATA_M0),
	.HREADY(HREADY_MST[0]),
	.HRESETn(HRESET_N),
	.HRESP(HRESP_M0),
	.HSIZE(HSIZE_ARR[2:0]),
	.HTRANS(HTRANS_ARR[1:0]),
	.HWDATA(HWDATA_ARR[31:0]),
	.HWRITE(HWRITE_ARR[0])
);

AHBMasterToArbiter
#(
	.P_MASTERS_NUM(P_MASTERS_NUM))
U10
(
	.HADDR(HADDR),
	.HADDR_ARR(HADDR_ARR[(P_MASTERS_NUM * 32) - 1:0]),
	.HBURST(HBURST),
	.HBURST_ARR(HBURST_ARR[(P_MASTERS_NUM * 3) - 1:0]),
	.HMASTER(HMASTER[clog2(P_MASTERS_NUM-1)-1:0]),
	.HTRANS(HTRANS),
	.HTRANS_ARR(HTRANS_ARR[(P_MASTERS_NUM * 2) - 1:0])
);

AHBSlaveDummy U11
(
	.HADDR(HADDR_SL0),
	.HBURST(HBURST_SL0),
	.HCLK(HCLK),
	.HPROT(HPROT_SL0),
	.HRDATA(HRDATA_ARR[31:0]),
	.HREADY(HREADY_ARR[0]),
	.HRESET_N(HRESET_N),
	.HRESP(HRESP_ARR[1:0]),
	.HSEL(HSEL_DUMMY),
	.HSIZE(HSIZE_SL0),
	.HTRANS(HTRANS_SL0),
	.HWDATA(HWDATA_SL0),
	.HWRITE(HWRITE_SL0)
);

AHBMasterToSlave U12
(
	.HADDR(HADDR_SL1),
	.HADDR_ARR(HADDR_ARR[(P_MASTERS_NUM * 32)-1:0]),
	.HBURST(HBURST_SL1),
	.HBURST_ARR(HBURST_ARR[(P_MASTERS_NUM * 3)-1:0]),
	.HCLK(HCLK),
	.HMASTER(HMASTER[clog2(P_MASTERS_NUM-1)-1:0]),
	.HMASTER_VALID(HMASTER_VALID),
	.HPROT(HPROT_SL1),
	.HPROT_ARR(HPROT_ARR[(P_MASTERS_NUM * 4)-1:0]),
	.HREADY(HREADY_ARR[1]),
	.HRESET_N(HRESET_N),
	.HSEL(HSEL[0]),
	.HSIZE(HSIZE_SL1),
	.HSIZE_ARR(HSIZE_ARR[(P_MASTERS_NUM * 3)-1:0]),
	.HTRANS(HTRANS_SL1),
	.HTRANS_ARR(HTRANS_ARR[(P_MASTERS_NUM * 2)-1:0]),
	.HTRANS_IN(HTRANS),
	.HWDATA(HWDATA_SL1),
	.HWDATA_ARR(HWDATA_ARR[(P_MASTERS_NUM * 32)-1:0]),
	.HWRITE(HWRITE_SL1),
	.HWRITE_ARR(HWRITE_ARR[P_MASTERS_NUM-1:0])
);

AHBMasterToSlave U13
(
	.HADDR(HADDR_SL2),
	.HADDR_ARR(HADDR_ARR[(P_MASTERS_NUM * 32)-1:0]),
	.HBURST(HBURST_SL2),
	.HBURST_ARR(HBURST_ARR[(P_MASTERS_NUM * 3)-1:0]),
	.HCLK(HCLK),
	.HMASTER(HMASTER[clog2(P_MASTERS_NUM-1)-1:0]),
	.HMASTER_VALID(HMASTER_VALID),
	.HPROT(HPROT_SL2),
	.HPROT_ARR(HPROT_ARR[(P_MASTERS_NUM * 4)-1:0]),
	.HREADY(HREADY_ARR[2]),
	.HRESET_N(HRESET_N),
	.HSEL(HSEL[1]),
	.HSIZE(HSIZE_SL2),
	.HSIZE_ARR(HSIZE_ARR[(P_MASTERS_NUM * 3)-1:0]),
	.HTRANS(HTRANS_SL2),
	.HTRANS_ARR(HTRANS_ARR[(P_MASTERS_NUM * 2)-1:0]),
	.HTRANS_IN(HTRANS),
	.HWDATA(HWDATA_SL2),
	.HWDATA_ARR(HWDATA_ARR[(P_MASTERS_NUM * 32)-1:0]),
	.HWRITE(HWRITE_SL2),
	.HWRITE_ARR(HWRITE_ARR[P_MASTERS_NUM-1:0])
);

AHBMasterToSlave U2
(
	.HADDR(HADDR_SL0),
	.HADDR_ARR(HADDR_ARR[(P_MASTERS_NUM * 32)-1:0]),
	.HBURST(HBURST_SL0),
	.HBURST_ARR(HBURST_ARR[(P_MASTERS_NUM * 3)-1:0]),
	.HCLK(HCLK),
	.HMASTER(HMASTER[clog2(P_MASTERS_NUM-1)-1:0]),
	.HMASTER_VALID(HMASTER_VALID),
	.HPROT(HPROT_SL0),
	.HPROT_ARR(HPROT_ARR[(P_MASTERS_NUM * 4)-1:0]),
	.HREADY(HREADY_ARR[0]),
	.HRESET_N(HRESET_N),
	.HSEL(HSEL_DUMMY),
	.HSIZE(HSIZE_SL0),
	.HSIZE_ARR(HSIZE_ARR[(P_MASTERS_NUM * 3)-1:0]),
	.HTRANS(HTRANS_SL0),
	.HTRANS_ARR(HTRANS_ARR[(P_MASTERS_NUM * 2)-1:0]),
	.HTRANS_IN(HTRANS),
	.HWDATA(HWDATA_SL0),
	.HWDATA_ARR(HWDATA_ARR[(P_MASTERS_NUM * 32)-1:0]),
	.HWRITE(HWRITE_SL0),
	.HWRITE_ARR(HWRITE_ARR[P_MASTERS_NUM-1:0])
);

AHBArbiter
#(
	.P_MASTERS_NUM(P_MASTERS_NUM),
	.P_SLAVES_NUM(P_SLAVES_NUM))
U3
(
	.HADDR(HADDR),
	.HBURST(HBURST),
	.HBUSREQ(HBUSREQ[P_MASTERS_NUM - 1:0]),
	.HCLK(HCLK),
	.HGRANT(HGRANT[P_MASTERS_NUM - 1:0]),
	.HLOCK(HLOCK[P_MASTERS_NUM - 1:0]),
	.HMASTER(HMASTER[clog2(P_MASTERS_NUM-1)-1:0]),
	.HMASTER_VALID(HMASTER_VALID),
	.HREADY_MST(HREADY_MST[P_MASTERS_NUM - 1:0]),
	.HREADY_SLV(HREADY_ARR[P_SLAVES_NUM:0]),
	.HRESETn(HRESET_N),
	.HSEL(HSEL[P_SLAVES_NUM-1:0]),
	.HTRANS(HTRANS)
);

AHBDecoder
#(
	.P_SLAVES_NUM(P_SLAVES_NUM))
U4
(
	.HADDR(HADDR),
	.HMASTER_VALID(HMASTER_VALID),
	.HSEL(HSEL[P_SLAVES_NUM-1:0]),
	.HSEL_DUMMY(HSEL_DUMMY),
	.HTRANS(HTRANS)
);

AHBSlaveToMaster
#(
	.P_MASTERS_NUM(P_MASTERS_NUM),
	.P_MASTER_ADDR(0),
	.P_SLAVES_NUM(P_SLAVES_NUM))
U5
(
	.HBUSREQ(HBUSREQ[P_MASTERS_NUM - 1:0]),
	.HCLK(HCLK),
	.HGRANT(HGRANT[P_MASTERS_NUM - 1:0]),
	.HMASTER(HMASTER[clog2(P_MASTERS_NUM-1)-1:0]),
	.HMASTER_VALID(HMASTER_VALID),
	.HRDATA_ARR(HRDATA_ARR[((P_SLAVES_NUM+1) * 32)-1:0]),
	.HRDATA_M(HRDATA_M0),
	.HREADY_M(HREADY_MST[0]),
	.HREADY_SLV(HREADY_ARR[P_SLAVES_NUM:0]),
	.HRESETn(HRESET_N),
	.HRESP_ARR(HRESP_ARR[((P_SLAVES_NUM+1) * 2)-1:0]),
	.HRESP_M(HRESP_M0),
	.HSEL(HSEL[P_SLAVES_NUM-1:0]),
	.HTRANS(HTRANS)
);

AHBMaster
#(
	.P_IN_FILENAME("input_2.txt"),
	.P_MASTER_NAME("1"),
	.P_OUT_FILENAME("output_2.txt"),
	.P_STOP_SIM_AT_EOF(1))
U6
(
	.HADDR(HADDR_ARR[63:32]),
	.HBURST(HBURST_ARR[5:3]),
	.HBUSREQ(HBUSREQ[1]),
	.HCLK(HCLK),
	.HGRANT(HGRANT[1]),
	.HLOCK(HLOCK[1]),
	.HPROT(HPROT_ARR[7:4]),
	.HRDATA(HRDATA_M1),
	.HREADY(HREADY_MST[1]),
	.HRESETn(HRESET_N),
	.HRESP(HRESP_M1),
	.HSIZE(HSIZE_ARR[5:3]),
	.HTRANS(HTRANS_ARR[3:2]),
	.HWDATA(HWDATA_ARR[63:32]),
	.HWRITE(HWRITE_ARR[1])
);

AHBSlaveToMaster
#(
	.P_MASTERS_NUM(P_MASTERS_NUM),
	.P_MASTER_ADDR(1),
	.P_SLAVES_NUM(P_SLAVES_NUM))
U7
(
	.HBUSREQ(HBUSREQ[P_MASTERS_NUM - 1:0]),
	.HCLK(HCLK),
	.HGRANT(HGRANT[P_MASTERS_NUM - 1:0]),
	.HMASTER(HMASTER[clog2(P_MASTERS_NUM-1)-1:0]),
	.HMASTER_VALID(HMASTER_VALID),
	.HRDATA_ARR(HRDATA_ARR[((P_SLAVES_NUM+1) * 32)-1:0]),
	.HRDATA_M(HRDATA_M1),
	.HREADY_M(HREADY_MST[1]),
	.HREADY_SLV(HREADY_ARR[P_SLAVES_NUM:0]),
	.HRESETn(HRESET_N),
	.HRESP_ARR(HRESP_ARR[((P_SLAVES_NUM+1) * 2)-1:0]),
	.HRESP_M(HRESP_M1),
	.HSEL(HSEL[P_SLAVES_NUM-1:0]),
	.HTRANS(HTRANS)
);

AHBSlave U8
(
	.HADDR(HADDR_SL1),
	.HBURST(HBURST_SL1),
	.HCLK(HCLK),
	.HPROT(HPROT_SL1),
	.HRDATA(HRDATA_ARR[63:32]),
	.HREADY(HREADY_ARR[1]),
	.HRESETn(HRESET_N),
	.HRESP(HRESP_ARR[3:2]),
	.HSEL(HSEL[0]),
	.HSIZE(HSIZE_SL1),
	.HTRANS(HTRANS_SL1),
	.HWDATA(HWDATA_SL1),
	.HWRITE(HWRITE_SL1)
);

AHBSlave U9
(
	.HADDR(HADDR_SL2),
	.HBURST(HBURST_SL2),
	.HCLK(HCLK),
	.HPROT(HPROT_SL2),
	.HRDATA(HRDATA_ARR[95:64]),
	.HREADY(HREADY_ARR[2]),
	.HRESETn(HRESET_N),
	.HRESP(HRESP_ARR[5:4]),
	.HSEL(HSEL[1]),
	.HSIZE(HSIZE_SL2),
	.HTRANS(HTRANS_SL2),
	.HWDATA(HWDATA_SL2),
	.HWRITE(HWRITE_SL2)
);

endmodule 
