//////////////////////////////////////////////////////////////////
//                                                             
// Copyright (C) 2010 Pulse Logic                       
//                    info@pulselogic.com.pl                   
//                                                             
// This source file may be used and distributed without        
// restriction provided that this copyright statement is not   
// removed from the file and that any derivative work contains 
// the original copyright notice and the associated disclaimer.
//                                                             
//     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
// POSSIBILITY OF SUCH DAMAGE.                                 
//                                                             
//////////////////////////////////////////////////////////////////
//
// Title       : AHBMasterToSlave
// Design      : AHB Testbench
// Company     : Pulse Logic
//
//////////////////////////////////////////////////////////////////
//
// File        : AHBMasterToSlave.v
// File Type   : RTL (Obfuscated code)
// Tool        : Xilinx WebPack
//
//////////////////////////////////////////////////////////////////
//
// Description : AHB Masters to Slave multiplexer.
//		 Allows connecting multiple AHB master modules to selected AHB slave. 
//////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module AHBMasterToSlave 
    #(
    parameter P_MASTERS_NUM = 2                          //Number of masters in the system
    )	
    (
    input wire HCLK,                                     //AHB Clock
    input wire HRESET_N,                                 //AHB Reset
    input wire HSEL,                                     //AHB HSEL signal
    input wire [1 : 0] HTRANS_IN,                        //AHB HTRANS from selected master. Connected to output of AHBMasterToArbiter module
    input wire [clog2(P_MASTERS_NUM-1)-1 : 0] HMASTER,   //AHB HMASTER signal
    input wire HMASTER_VALID,                            //AHB HMASTER valid signal from arbiter
    input wire HREADY,                                   //AHB HREADY signal
	
    input wire [(P_MASTERS_NUM * 2)-1 : 0] HTRANS_ARR,   //Array of AHB HATRANS from all masters
    input wire [(P_MASTERS_NUM * 3)-1 : 0] HSIZE_ARR,    //Array of AHB HSIZE from all masters
    input wire [(P_MASTERS_NUM * 3)-1 : 0] HBURST_ARR,   //Array of AHB HBURST from all masters
    input wire [(P_MASTERS_NUM * 4)-1 : 0] HPROT_ARR,    //Array of AHB HPROT from all masters
    input wire [(P_MASTERS_NUM * 32)-1 : 0] HADDR_ARR,   //Array of AHB HADDR from all masters
    input wire [(P_MASTERS_NUM * 32)-1 : 0] HWDATA_ARR,  //Array of AHB HWDATA from all masters
    input wire [P_MASTERS_NUM-1 : 0] HWRITE_ARR,         //Array of AHB HWRITE from all masters
	  
    output wire [1 : 0] HTRANS,                          //AHB HTRANS from master that is requesting access to the slave.
    output wire [2 : 0] HSIZE,                           //AHB HSIZE from master that is requesting access to the slave.
    output wire [2 : 0] HBURST,                          //AHB HBURST from master that is requesting access to the slave.
    output wire [3 : 0] HPROT,                           //AHB HPROT from master that is requesting access to the slave.
    output wire [31 : 0] HADDR,                          //AHB HADDR from master that is requesting access to the slave.
    output wire [31 : 0] HWDATA,                         //AHB HWDATA from master that is requesting access to the slave.
    output wire HWRITE                                   //AHB HWRITE from master that is requesting access to the slave.
    );

localparam ID_S_2b08f1c5_54955117_E = 2'b00; localparam ID_S_1d7ead3b_19138270_E = 2'b10; localparam ID_S_cf0ec30_2e07c95e_E =  2'b11; localparam ID_S_2b05646a_36485dfd_E = 2'b01;  localparam ID_S_28bbb329_3d88d889_E = 3'b000; localparam ID_S_2b091b33_54bd742c_E = 3'b001; localparam ID_S_d2c0195_4ee7bce8_E = 3'b010; localparam ID_S_c2c81c7_7090cc9d_E = 3'b011; localparam ID_S_d2c0199_4ee7bce4_E = 3'b100; localparam ID_S_c2c81cb_7090cc91_E = 3'b101; localparam ID_S_32ac3408_728816c3_E = 3'b110; localparam ID_S_11bcba7a_a8f7535_E = 3'b111;  localparam ID_S_bab60d1_5cb64e54_E = 3'b000;  localparam ID_S_1177a40_13deebe6_E = 3'b001;  localparam ID_S_1177a7e_13defbe0_E = 3'b010; localparam ID_S_1177ae3_13ded3e3_E = 3'b011;  localparam ID_S_2406c1f4_7ed0c7a6_E = 3'b100;  localparam ID_S_2406c696_7edcdeac_E = 3'b101;  localparam ID_S_2406d2d1_7ec0e02b_E = 3'b110;  localparam ID_S_24def760_78eccf50_E = 3'b111;   localparam ID_S_2b0c587b_15198a1a_E = 2'b00; localparam ID_S_be690d1_b1c9322_E = 2'b01;  localparam ID_S_ccab75d_1a31786_E = 2'b10; localparam ID_S_ce2b4d3_2b954867_E = 2'b11;   localparam ID_S_2b0de403_328a5972_E = 1'b0; localparam ID_S_d2c2432_4ec7f294_E = 1'b1;    localparam ID_S_59dd33e1_20a57556_E = 0;   function integer ID_S_609e2258_1c7667c_E (input [2 : 0] ID_S_f2e7e15_72f12829_E);    integer ID_S_b88a54f_7e3417e5_E; begin   case (ID_S_f2e7e15_72f12829_E)   ID_S_28bbb329_3d88d889_E , ID_S_2b091b33_54bd742c_E : ID_S_b88a54f_7e3417e5_E = 1;   ID_S_d2c0195_4ee7bce8_E  , ID_S_c2c81c7_7090cc9d_E :   ID_S_b88a54f_7e3417e5_E = 4;   ID_S_d2c0199_4ee7bce4_E  , ID_S_c2c81cb_7090cc91_E :   ID_S_b88a54f_7e3417e5_E = 8;   ID_S_32ac3408_728816c3_E , ID_S_11bcba7a_a8f7535_E :  ID_S_b88a54f_7e3417e5_E = 16;  endcase  ID_S_609e2258_1c7667c_E = ID_S_b88a54f_7e3417e5_E; end endfunction    function integer clog2 (input integer ID_S_1020db36_1f410d8f_E);    integer ID_S_7c9a2e19_19de53b5_E; begin    for(ID_S_7c9a2e19_19de53b5_E = 0; ID_S_1020db36_1f410d8f_E > 0; ID_S_7c9a2e19_19de53b5_E = ID_S_7c9a2e19_19de53b5_E + 1)  begin   ID_S_1020db36_1f410d8f_E = ID_S_1020db36_1f410d8f_E >> 1;  end  clog2 = ID_S_7c9a2e19_19de53b5_E; end endfunction   reg [clog2(P_MASTERS_NUM-1)-1 : 0] ID_S_817f08c_4e196cfd_E; wire [clog2(P_MASTERS_NUM-1)-1 : 0] ID_S_521a4ec2_774ebb2_E;   always @(negedge HRESET_N or posedge HCLK) begin  if(HRESET_N == 1'b0)   ID_S_817f08c_4e196cfd_E <= 0;  else    if(HSEL == 1'b1 && HMASTER_VALID == 1'b1 && HREADY == 1'b1 && HTRANS_IN != ID_S_2b08f1c5_54955117_E)     ID_S_817f08c_4e196cfd_E <= HMASTER;  end    assign ID_S_521a4ec2_774ebb2_E = (HSEL == 1'b1 && HMASTER_VALID == 1'b1 && (HTRANS_IN != ID_S_2b08f1c5_54955117_E) && HREADY == 1'b1) ? HMASTER : ID_S_817f08c_4e196cfd_E;  wire [1 : 0] ID_S_6217899_1bfce5d6_E [P_MASTERS_NUM-1 : 0]; wire [2 : 0] ID_S_18d6520c_53be6766_E [P_MASTERS_NUM-1 : 0]; wire [2 : 0] ID_S_4280f1a1_13061f36_E [P_MASTERS_NUM-1 : 0]; wire [3 : 0] ID_S_7043a6d6_2b1ac649_E [P_MASTERS_NUM-1 : 0]; wire [31 : 0] ID_S_48ebeb2c_5d1a31f3_E [P_MASTERS_NUM-1 : 0]; wire [31 : 0] ID_S_4b66202_651d9de8_E [P_MASTERS_NUM-1 : 0]; wire [P_MASTERS_NUM-1 : 0] ID_S_586a4e5c_5c9ab06c_E;    genvar ID_S_7ca037e1_7fc5931a_E;  generate    for(ID_S_7ca037e1_7fc5931a_E = 0; ID_S_7ca037e1_7fc5931a_E < P_MASTERS_NUM; ID_S_7ca037e1_7fc5931a_E = ID_S_7ca037e1_7fc5931a_E + 1)  begin : ID_S_b88bc32_7e2033e2_E   assign ID_S_6217899_1bfce5d6_E[ID_S_7ca037e1_7fc5931a_E] = HTRANS_ARR[(2 * (ID_S_7ca037e1_7fc5931a_E + 1) - 1) : (2 * ID_S_7ca037e1_7fc5931a_E)];   assign ID_S_18d6520c_53be6766_E[ID_S_7ca037e1_7fc5931a_E] = HSIZE_ARR[(3 * (ID_S_7ca037e1_7fc5931a_E + 1) - 1) : (3 * ID_S_7ca037e1_7fc5931a_E)];   assign ID_S_4280f1a1_13061f36_E[ID_S_7ca037e1_7fc5931a_E] = HBURST_ARR[(3 * (ID_S_7ca037e1_7fc5931a_E + 1) - 1) : (3 * ID_S_7ca037e1_7fc5931a_E)];   assign ID_S_7043a6d6_2b1ac649_E[ID_S_7ca037e1_7fc5931a_E] = HPROT_ARR[(4 * (ID_S_7ca037e1_7fc5931a_E + 1) - 1) : (4 * ID_S_7ca037e1_7fc5931a_E)];   assign ID_S_48ebeb2c_5d1a31f3_E[ID_S_7ca037e1_7fc5931a_E] = HADDR_ARR[(32 * (ID_S_7ca037e1_7fc5931a_E + 1) - 1) : (32 * ID_S_7ca037e1_7fc5931a_E)];   assign ID_S_4b66202_651d9de8_E[ID_S_7ca037e1_7fc5931a_E] = HWDATA_ARR[(32 * (ID_S_7ca037e1_7fc5931a_E + 1) - 1) : (32 * ID_S_7ca037e1_7fc5931a_E)];     assign ID_S_586a4e5c_5c9ab06c_E[ID_S_7ca037e1_7fc5931a_E] = HWRITE_ARR[ID_S_7ca037e1_7fc5931a_E];  end endgenerate  assign HTRANS = ID_S_6217899_1bfce5d6_E[ID_S_521a4ec2_774ebb2_E]; assign HSIZE =  ID_S_18d6520c_53be6766_E[ID_S_521a4ec2_774ebb2_E]; assign HBURST = ID_S_4280f1a1_13061f36_E[ID_S_521a4ec2_774ebb2_E]; assign HPROT = ID_S_7043a6d6_2b1ac649_E[ID_S_521a4ec2_774ebb2_E]; assign HADDR = ID_S_48ebeb2c_5d1a31f3_E[ID_S_521a4ec2_774ebb2_E]; assign HWDATA = ID_S_4b66202_651d9de8_E[ID_S_521a4ec2_774ebb2_E]; assign HWRITE = ID_S_586a4e5c_5c9ab06c_E[ID_S_521a4ec2_774ebb2_E];   endmodule 