//////////////////////////////////////////////////////////////////
//                                                             
// Copyright (C) 2010 Pulse Logic                       
//                    info@pulselogic.com.pl                   
//                                                             
// This source file may be used and distributed without        
// restriction provided that this copyright statement is not   
// removed from the file and that any derivative work contains 
// the original copyright notice and the associated disclaimer.
//                                                             
//     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
// POSSIBILITY OF SUCH DAMAGE.                                 
//                                                             
//////////////////////////////////////////////////////////////////
//
// Title       : AHBSlave
// Design      : AHB Testbench
// Company     : Pulse Logic
//
//////////////////////////////////////////////////////////////////
//
// File        : AHBSlave.v
// File Type   : RTL (Obfuscated code)
// Tool        : Xilinx WebPack
//
//////////////////////////////////////////////////////////////////
//
//Description: AHB Slave module. This is synchronous RAM memory with AHB interface.
//	
//////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps
	
module AHBSlave 
    #(
    parameter P_ADDR_SIZE = 1024,     //Address space occupied by slave.
    parameter P_WAIT_STATES = 5       //Number of inserted wait state cycles 
    )	
    (
    input wire HCLK,                  //Clock
    input wire HRESETn,               //Reset
    input wire HSEL,                  //AHB HSEL signal
    input wire [1 : 0] HTRANS,        //AHB HTRANS signal
    input wire [2 : 0] HBURST,        //AHB HBURST signal
    input wire [2 : 0] HSIZE,         //AHB HSIZE signal
    input wire [31 : 0] HADDR,        //AHB HADDR signal
    input wire [3 : 0] HPROT,         //AHB HPROT signal
    input wire [31 : 0] HWDATA,       //AHB HWDATA signal
    input wire HWRITE,                //AHB HWRITE signal
    output wire [31 : 0] HRDATA,      //AHB HRDATA signal
    output wire [1 : 0] HRESP,        //AHB HRESP signal
    output wire HREADY                //AHB HREADY signal
    );

localparam ID_S_2b08f1c5_54955117_E = 2'b00; localparam ID_S_1d7ead3b_19138270_E = 2'b10; localparam ID_S_cf0ec30_2e07c95e_E =  2'b11; localparam ID_S_2b05646a_36485dfd_E = 2'b01;  localparam ID_S_28bbb329_3d88d889_E = 3'b000; localparam ID_S_2b091b33_54bd742c_E = 3'b001; localparam ID_S_d2c0195_4ee7bce8_E = 3'b010; localparam ID_S_c2c81c7_7090cc9d_E = 3'b011; localparam ID_S_d2c0199_4ee7bce4_E = 3'b100; localparam ID_S_c2c81cb_7090cc91_E = 3'b101; localparam ID_S_32ac3408_728816c3_E = 3'b110; localparam ID_S_11bcba7a_a8f7535_E = 3'b111;  localparam ID_S_bab60d1_5cb64e54_E = 3'b000;  localparam ID_S_1177a40_13deebe6_E = 3'b001;  localparam ID_S_1177a7e_13defbe0_E = 3'b010; localparam ID_S_1177ae3_13ded3e3_E = 3'b011;  localparam ID_S_2406c1f4_7ed0c7a6_E = 3'b100;  localparam ID_S_2406c696_7edcdeac_E = 3'b101;  localparam ID_S_2406d2d1_7ec0e02b_E = 3'b110;  localparam ID_S_24def760_78eccf50_E = 3'b111;   localparam ID_S_2b0c587b_15198a1a_E = 2'b00; localparam ID_S_be690d1_b1c9322_E = 2'b01;  localparam ID_S_ccab75d_1a31786_E = 2'b10; localparam ID_S_ce2b4d3_2b954867_E = 2'b11;   localparam ID_S_2b0de403_328a5972_E = 1'b0; localparam ID_S_d2c2432_4ec7f294_E = 1'b1;    localparam ID_S_59dd33e1_20a57556_E = 0;   function integer ID_S_609e2258_1c7667c_E (input [2 : 0] ID_S_f2e7e15_72f12829_E);    integer ID_S_b88a54f_7e3417e5_E; begin   case (ID_S_f2e7e15_72f12829_E)   ID_S_28bbb329_3d88d889_E , ID_S_2b091b33_54bd742c_E : ID_S_b88a54f_7e3417e5_E = 1;   ID_S_d2c0195_4ee7bce8_E  , ID_S_c2c81c7_7090cc9d_E :   ID_S_b88a54f_7e3417e5_E = 4;   ID_S_d2c0199_4ee7bce4_E  , ID_S_c2c81cb_7090cc91_E :   ID_S_b88a54f_7e3417e5_E = 8;   ID_S_32ac3408_728816c3_E , ID_S_11bcba7a_a8f7535_E :  ID_S_b88a54f_7e3417e5_E = 16;  endcase  ID_S_609e2258_1c7667c_E = ID_S_b88a54f_7e3417e5_E; end endfunction    function integer ID_S_f3b989c_40d222f2_E (input integer ID_S_1020db36_1f410d8f_E);    integer ID_S_7c9a2e19_19de53b5_E; begin    for(ID_S_7c9a2e19_19de53b5_E = 0; ID_S_1020db36_1f410d8f_E > 0; ID_S_7c9a2e19_19de53b5_E = ID_S_7c9a2e19_19de53b5_E + 1)  begin   ID_S_1020db36_1f410d8f_E = ID_S_1020db36_1f410d8f_E >> 1;  end  ID_S_f3b989c_40d222f2_E = ID_S_7c9a2e19_19de53b5_E; end endfunction   localparam ID_S_68f6904e_64d4c711_E = 1; localparam ID_S_1f0337c_6531c275_E = 1 << 1;   function [31 : 0] ID_S_20690f9_73e0b666_E(input [2 : 0] ID_S_7c9dede0_7efd6739_E, input [31 : 0] ID_S_7b3c3768_48fe117d_E, input [31 : 0] ID_S_509d483a_310124e3_E, input [1 : 0] ID_S_7c93f700_3adbbec0_E); begin  case (ID_S_7c9dede0_7efd6739_E)   ID_S_bab60d1_5cb64e54_E :     case(ID_S_7c93f700_3adbbec0_E)     2'b00 : ID_S_20690f9_73e0b666_E = {ID_S_509d483a_310124e3_E[31 : 8], ID_S_7b3c3768_48fe117d_E[7 : 0]};     2'b01 : ID_S_20690f9_73e0b666_E = {ID_S_509d483a_310124e3_E[31 : 16], ID_S_7b3c3768_48fe117d_E[15 : 8], ID_S_509d483a_310124e3_E[7 : 0]};     2'b10 : ID_S_20690f9_73e0b666_E = {ID_S_509d483a_310124e3_E[31 : 24], ID_S_7b3c3768_48fe117d_E[23 : 16], ID_S_509d483a_310124e3_E[15 : 0]};     2'b11 : ID_S_20690f9_73e0b666_E = {ID_S_7b3c3768_48fe117d_E[31 : 24], ID_S_509d483a_310124e3_E[23 : 0]};     default : ID_S_20690f9_73e0b666_E = {ID_S_509d483a_310124e3_E[31 : 8], ID_S_7b3c3768_48fe117d_E[7 : 0]};    endcase   ID_S_1177a40_13deebe6_E :    if(ID_S_7c93f700_3adbbec0_E[1] == 1'b0)     ID_S_20690f9_73e0b666_E = {ID_S_509d483a_310124e3_E[31 : 16], ID_S_7b3c3768_48fe117d_E[15 : 0]};    else     ID_S_20690f9_73e0b666_E = {ID_S_7b3c3768_48fe117d_E[31 : 16], ID_S_509d483a_310124e3_E[15 : 0]};   ID_S_1177a7e_13defbe0_E :     ID_S_20690f9_73e0b666_E = ID_S_7b3c3768_48fe117d_E;    default : ID_S_20690f9_73e0b666_E = ID_S_7b3c3768_48fe117d_E;  endcase end endfunction  reg [31 : 0] ID_S_3dcefc61_7f532e29_E [P_ADDR_SIZE-1 : 0];   integer ID_S_1fbf30ea_365eecc3_E;        integer ID_S_1498e763_437e974b_E;          reg ID_S_6f9276e6_1c29c48f_E;          reg [ID_S_f3b989c_40d222f2_E(P_ADDR_SIZE)+1 : 0] ID_S_705b2db8_31342c04_E;  wire [ID_S_f3b989c_40d222f2_E(P_ADDR_SIZE)+1 : 0] ID_S_7bc3b571_4f7a322f_E;  reg [31 : 0] ID_S_2ff312_2b5e1b5_E;       wire [31 : 0] ID_S_36479354_63661788_E;        wire ID_S_283095d6_3ff907ae_E;          reg [2 : 0] ID_S_1ccf1145_5fe23f84_E;        reg ID_S_5db531b5_5e01c0df_E;           integer ID_S_ce4dd24_42c45fb5_E; wire ID_S_c34822c_2998bf49_E;           always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   begin    for(ID_S_ce4dd24_42c45fb5_E = 0; ID_S_ce4dd24_42c45fb5_E < P_ADDR_SIZE; ID_S_ce4dd24_42c45fb5_E = ID_S_ce4dd24_42c45fb5_E + 1)     ID_S_3dcefc61_7f532e29_E[ID_S_ce4dd24_42c45fb5_E] <= 32'h00000123;    ID_S_2ff312_2b5e1b5_E <= {32{1'b0}};   end   else   begin    if(ID_S_283095d6_3ff907ae_E == 1'b1)     ID_S_3dcefc61_7f532e29_E[ID_S_7bc3b571_4f7a322f_E[ID_S_f3b989c_40d222f2_E(P_ADDR_SIZE)+1 : 2]] = ID_S_36479354_63661788_E;         ID_S_2ff312_2b5e1b5_E <= ID_S_3dcefc61_7f532e29_E[ID_S_7bc3b571_4f7a322f_E[ID_S_f3b989c_40d222f2_E(P_ADDR_SIZE)+1 : 2]];    end end   always @(negedge HRESETn or posedge HCLK) begin  if(HRESETn == 1'b0)   begin    ID_S_1498e763_437e974b_E <= ID_S_68f6904e_64d4c711_E;    ID_S_705b2db8_31342c04_E <= {32{1'b0}};    ID_S_1fbf30ea_365eecc3_E <= 0;    ID_S_6f9276e6_1c29c48f_E <= 1'b0;     ID_S_5db531b5_5e01c0df_E <= 1'b0;   end   else   begin    case(ID_S_1498e763_437e974b_E)     ID_S_68f6904e_64d4c711_E :       begin       if(HSEL == 1'b1 && HTRANS != ID_S_2b08f1c5_54955117_E)         begin         ID_S_1ccf1145_5fe23f84_E <= HSIZE;         ID_S_5db531b5_5e01c0df_E <= HWRITE;         ID_S_705b2db8_31342c04_E <= HADDR[ID_S_f3b989c_40d222f2_E(P_ADDR_SIZE) : 0];         ID_S_1498e763_437e974b_E <= ID_S_1f0337c_6531c275_E;         ID_S_1fbf30ea_365eecc3_E <= 0;        end        ID_S_6f9276e6_1c29c48f_E <= 1'b0;       end     ID_S_1f0337c_6531c275_E :       begin             if(ID_S_1fbf30ea_365eecc3_E == P_WAIT_STATES - 1)       begin         ID_S_1498e763_437e974b_E <= ID_S_68f6904e_64d4c711_E;        ID_S_6f9276e6_1c29c48f_E <= 1'b1;       end        else        ID_S_1fbf30ea_365eecc3_E <= ID_S_1fbf30ea_365eecc3_E + 1;             end     default : ID_S_1498e763_437e974b_E <= ID_S_68f6904e_64d4c711_E;    endcase       end end  assign ID_S_7bc3b571_4f7a322f_E = ID_S_705b2db8_31342c04_E; assign ID_S_36479354_63661788_E = ID_S_20690f9_73e0b666_E(ID_S_1ccf1145_5fe23f84_E, HWDATA, ID_S_2ff312_2b5e1b5_E, ID_S_705b2db8_31342c04_E[1 : 0]); assign ID_S_283095d6_3ff907ae_E = (ID_S_1498e763_437e974b_E == ID_S_1f0337c_6531c275_E && ID_S_5db531b5_5e01c0df_E == 1'b1) ? 1'b1 : 1'b0;  assign ID_S_c34822c_2998bf49_E = (ID_S_1498e763_437e974b_E == ID_S_68f6904e_64d4c711_E && ((HSEL == 1'b1) || ID_S_6f9276e6_1c29c48f_E == 1'b1)) ? 1'b1 : 1'b0; assign HRDATA = (ID_S_6f9276e6_1c29c48f_E == 1'b1 || (ID_S_1498e763_437e974b_E == ID_S_1f0337c_6531c275_E)) ? (ID_S_5db531b5_5e01c0df_E == 1'b0 && ID_S_c34822c_2998bf49_E == 1'b1 ? ID_S_2ff312_2b5e1b5_E : (ID_S_6f9276e6_1c29c48f_E == 1'b1 ? 32'h00000000 : 32'h01234567)) :     (ID_S_1498e763_437e974b_E == ID_S_68f6904e_64d4c711_E && (HSEL == 1'b1 && HTRANS != ID_S_2b08f1c5_54955117_E)) ? 32'h456789AB :  32'hABCDEF01;  assign HREADY = ID_S_c34822c_2998bf49_E;  assign HRESP = 2'b00; endmodule 