//////////////////////////////////////////////////////////////////
//                                                             
// Copyright (C) 2010 Pulse Logic                       
//                    info@pulselogic.com.pl                   
//                                                             
// This source file may be used and distributed without        
// restriction provided that this copyright statement is not   
// removed from the file and that any derivative work contains 
// the original copyright notice and the associated disclaimer.
//                                                             
//     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
// POSSIBILITY OF SUCH DAMAGE.                                 
//                                                             
//////////////////////////////////////////////////////////////////
//
// Title       : FileRead
// Design      : AHB Testbench
// Company     : Pulse Logic
//
//////////////////////////////////////////////////////////////////
//
// File        : FileRead.v
// File Type   : RTL (Obfuscated code)
// Tool        : Xilinx WebPack
//
//////////////////////////////////////////////////////////////////
//
// Description : Module for reading and parsing input file with definition of AHB bus cycles
//	
//////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module FileRead
	#(
	parameter fileName = "input.txt"                //Input file name with 
	)
	(	
	output wire CMD,                                //Command or Data line detected.
	output wire [2 : 0] SIZE,                       //AHB HSIZE
	output wire [2 : 0] BURST,                      //AHB HBURST
	output wire [9 : 0] LEN,                        //Length of the transfer for INCR bursts.
	output wire [3 : 0] PROT,                       //AHB HPROT
	output wire [31 : 0] ADDR,                      //AHB HADDR
	output wire [31 : 0] DATA,                      //AHB HWDATA
	output wire [31 : 0] BREQ_DLY,                  //Integer defining delay in number of clock cycles to next bus request.
	output wire CMDREADDONE,                        //Command line read done
	output wire DATAREADDONE,                       //Data line read done
	output wire ENDOFFILE                           //End of input file reached
	);
    
localparam ID_S_2b08f1c5_54955117_E = 2'b00; localparam ID_S_1d7ead3b_19138270_E = 2'b10; localparam ID_S_cf0ec30_2e07c95e_E =  2'b11; localparam ID_S_2b05646a_36485dfd_E = 2'b01;  localparam ID_S_28bbb329_3d88d889_E = 3'b000; localparam ID_S_2b091b33_54bd742c_E = 3'b001; localparam ID_S_d2c0195_4ee7bce8_E = 3'b010; localparam ID_S_c2c81c7_7090cc9d_E = 3'b011; localparam ID_S_d2c0199_4ee7bce4_E = 3'b100; localparam ID_S_c2c81cb_7090cc91_E = 3'b101; localparam ID_S_32ac3408_728816c3_E = 3'b110; localparam ID_S_11bcba7a_a8f7535_E = 3'b111;  localparam ID_S_bab60d1_5cb64e54_E = 3'b000;  localparam ID_S_1177a40_13deebe6_E = 3'b001;  localparam ID_S_1177a7e_13defbe0_E = 3'b010; localparam ID_S_1177ae3_13ded3e3_E = 3'b011;  localparam ID_S_2406c1f4_7ed0c7a6_E = 3'b100;  localparam ID_S_2406c696_7edcdeac_E = 3'b101;  localparam ID_S_2406d2d1_7ec0e02b_E = 3'b110;  localparam ID_S_24def760_78eccf50_E = 3'b111;   localparam ID_S_2b0c587b_15198a1a_E = 2'b00; localparam ID_S_be690d1_b1c9322_E = 2'b01;  localparam ID_S_ccab75d_1a31786_E = 2'b10; localparam ID_S_ce2b4d3_2b954867_E = 2'b11;   localparam ID_S_2b0de403_328a5972_E = 1'b0; localparam ID_S_d2c2432_4ec7f294_E = 1'b1;    localparam ID_S_59dd33e1_20a57556_E = 0;   function integer ID_S_609e2258_1c7667c_E (input [2 : 0] ID_S_f2e7e15_72f12829_E);    integer ID_S_b88a54f_7e3417e5_E; begin   case (ID_S_f2e7e15_72f12829_E)   ID_S_28bbb329_3d88d889_E , ID_S_2b091b33_54bd742c_E : ID_S_b88a54f_7e3417e5_E = 1;   ID_S_d2c0195_4ee7bce8_E  , ID_S_c2c81c7_7090cc9d_E :   ID_S_b88a54f_7e3417e5_E = 4;   ID_S_d2c0199_4ee7bce4_E  , ID_S_c2c81cb_7090cc91_E :   ID_S_b88a54f_7e3417e5_E = 8;   ID_S_32ac3408_728816c3_E , ID_S_11bcba7a_a8f7535_E :  ID_S_b88a54f_7e3417e5_E = 16;  endcase  ID_S_609e2258_1c7667c_E = ID_S_b88a54f_7e3417e5_E; end endfunction    function integer ID_S_f3b989c_40d222f2_E (input integer ID_S_1020db36_1f410d8f_E);    integer ID_S_7c9a2e19_19de53b5_E; begin    for(ID_S_7c9a2e19_19de53b5_E = 0; ID_S_1020db36_1f410d8f_E > 0; ID_S_7c9a2e19_19de53b5_E = ID_S_7c9a2e19_19de53b5_E + 1)  begin   ID_S_1020db36_1f410d8f_E = ID_S_1020db36_1f410d8f_E >> 1;  end  ID_S_f3b989c_40d222f2_E = ID_S_7c9a2e19_19de53b5_E; end endfunction         localparam ID_S_953ec1e_474b34ca_E = 8;      integer ID_S_4bb65bc_60011761_E;  integer ID_S_b886fae_7e68b8e4_E;    reg [10*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_3c14e757_22cb98c3_E;  reg [6*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_76ba7519_4a223c2d_E;  reg [9*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_15ef1720_20c62687_E;  reg [9*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_2d92b735_2821d1c7_E;  reg [8*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_b2d9a84_4d0da2c3_E;  reg [6*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_433ffaaa_43a182bc_E;  reg [9*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_716d474e_50f0a81b_E;  reg [12*ID_S_953ec1e_474b34ca_E-1 : 0] ID_S_cda0def_29dede70_E;    reg ID_S_b886679_7e708268_E;  reg [2 : 0] ID_S_7c9dede0_7efd6739_E;  reg [2 : 0] ID_S_f2e7e15_72f12829_E;  reg [9 : 0] ID_S_b888bc4_7e4cf6e5_E;  reg [3 : 0] ID_S_7c9c6daa_1e49609e_E;  reg [31 : 0] ID_S_7c93f700_3adbbec0_E;  reg [31 : 0] ID_S_7c95915f_1ba7b2ab_E;   reg [31 : 0] ID_S_2a693d5e_6775b763_E;    event ID_S_793f7fc9_14f4cd11_E;  event ID_S_793b3647_16adf866_E;  reg ID_S_2cad1f9b_48e7e58e_E;  reg ID_S_6d11d081_2ba6b70c_E;   reg ID_S_3ce3d0b1_118095d6_E;    assign CMD = ID_S_b886679_7e708268_E;  assign SIZE = ID_S_7c9dede0_7efd6739_E;  assign BURST = ID_S_f2e7e15_72f12829_E;  assign LEN = ID_S_b888bc4_7e4cf6e5_E;  assign PROT = ID_S_7c9c6daa_1e49609e_E;  assign ADDR = ID_S_7c93f700_3adbbec0_E;  assign DATA = ID_S_7c95915f_1ba7b2ab_E;  assign BREQ_DLY = ID_S_2a693d5e_6775b763_E;  assign CMDREADDONE = ID_S_2cad1f9b_48e7e58e_E;  assign DATAREADDONE = ID_S_6d11d081_2ba6b70c_E;  assign ENDOFFILE = ID_S_3ce3d0b1_118095d6_E;    initial   begin    ID_S_b886679_7e708268_E = 0;    ID_S_7c9dede0_7efd6739_E = 0;    ID_S_f2e7e15_72f12829_E = 0;    ID_S_b888bc4_7e4cf6e5_E = 0;    ID_S_7c9c6daa_1e49609e_E = 0;    ID_S_7c93f700_3adbbec0_E = 0;    ID_S_7c95915f_1ba7b2ab_E = 0;       ID_S_2cad1f9b_48e7e58e_E = 0;       ID_S_6d11d081_2ba6b70c_E = 0;        ID_S_3ce3d0b1_118095d6_E = 0;        ID_S_4bb65bc_60011761_E = $fopen(fileName, "r");         end         always @(ID_S_793f7fc9_14f4cd11_E)     begin       ID_S_3ce3d0b1_118095d6_E = 1'b0;    ID_S_b886fae_7e68b8e4_E = $fscanf(ID_S_4bb65bc_60011761_E,"%s", ID_S_3c14e757_22cb98c3_E);        if( &ID_S_b886fae_7e68b8e4_E == 1'b1)     begin        ID_S_3ce3d0b1_118095d6_E = 1'b1;      ID_S_2cad1f9b_48e7e58e_E = ~ ID_S_2cad1f9b_48e7e58e_E;       ID_S_6d11d081_2ba6b70c_E = ~ ID_S_6d11d081_2ba6b70c_E;           ->ID_S_793b3647_16adf866_E;      $display("INFO : END OF FILE REACHED.");           end    else     begin      if(ID_S_3c14e757_22cb98c3_E == "COMMAND:")       begin                ID_S_b886fae_7e68b8e4_E = $fscanf(ID_S_4bb65bc_60011761_E,"%s %s %s %s %d %s %b %s %h %s %d", ID_S_76ba7519_4a223c2d_E, ID_S_15ef1720_20c62687_E, ID_S_2d92b735_2821d1c7_E, ID_S_b2d9a84_4d0da2c3_E, ID_S_b888bc4_7e4cf6e5_E, ID_S_433ffaaa_43a182bc_E, ID_S_7c9c6daa_1e49609e_E, ID_S_716d474e_50f0a81b_E, ID_S_7c93f700_3adbbec0_E, ID_S_cda0def_29dede70_E, ID_S_2a693d5e_6775b763_E);                 if( &ID_S_b886fae_7e68b8e4_E == 1'b1)        begin         $display("FATAL: UNKNOWN FILE FORMAT.");         $stop;        end                 case (ID_S_76ba7519_4a223c2d_E)         "READ" :  ID_S_b886679_7e708268_E = ID_S_2b0de403_328a5972_E;         "WRITE" : ID_S_b886679_7e708268_E = ID_S_d2c2432_4ec7f294_E;         default :                 begin           $display("FATAL: UNKNOWN FILE FORMAT. EXPECTING: READ, WRITE.");           $stop;          end        endcase                 case (ID_S_15ef1720_20c62687_E)         "BITS8" : ID_S_7c9dede0_7efd6739_E = ID_S_bab60d1_5cb64e54_E;         "BITS16" : ID_S_7c9dede0_7efd6739_E = ID_S_1177a40_13deebe6_E;         "BITS32" : ID_S_7c9dede0_7efd6739_E = ID_S_1177a7e_13defbe0_E;         "BITS64" : ID_S_7c9dede0_7efd6739_E = ID_S_1177ae3_13ded3e3_E;         "BITS128" : ID_S_7c9dede0_7efd6739_E = ID_S_2406c1f4_7ed0c7a6_E;         "BITS256" : ID_S_7c9dede0_7efd6739_E = ID_S_2406c696_7edcdeac_E;         "BITS512" : ID_S_7c9dede0_7efd6739_E = ID_S_2406d2d1_7ec0e02b_E;         "BITS1024" : ID_S_7c9dede0_7efd6739_E = ID_S_24def760_78eccf50_E;         default :                 begin           $display("FATAL: UNKNOWN FILE FORMAT. EXPECTING: BITS8, BITS16, BITS32, BITS64, BITS128, BITS256, BITS512, BITS1024.");           $stop;          end                endcase                case (ID_S_2d92b735_2821d1c7_E)         "SINGLE" : ID_S_f2e7e15_72f12829_E = ID_S_28bbb329_3d88d889_E;          "INCR" : ID_S_f2e7e15_72f12829_E = ID_S_2b091b33_54bd742c_E;          "WRAP4" : ID_S_f2e7e15_72f12829_E = ID_S_d2c0195_4ee7bce8_E;          "INCR4" : ID_S_f2e7e15_72f12829_E = ID_S_c2c81c7_7090cc9d_E;          "WRAP8" : ID_S_f2e7e15_72f12829_E = ID_S_d2c0199_4ee7bce4_E;          "INCR8" : ID_S_f2e7e15_72f12829_E = ID_S_c2c81cb_7090cc91_E;          "WRAP16" : ID_S_f2e7e15_72f12829_E = ID_S_32ac3408_728816c3_E;          "INCR16" : ID_S_f2e7e15_72f12829_E = ID_S_11bcba7a_a8f7535_E;         default :                 begin           $display("FATAL: UNKNOWN FILE FORMAT. EXPECTING: SINGLE, INCR, WRAP4, INCR4, WRAP8, INCR8, WRAP16, INCR16.");           $stop;          end                endcase                  ID_S_2cad1f9b_48e7e58e_E = ~ ID_S_2cad1f9b_48e7e58e_E;        ->ID_S_793b3647_16adf866_E;       end      else if(ID_S_3c14e757_22cb98c3_E == "DATA:")       begin                ID_S_b886fae_7e68b8e4_E = $fscanf(ID_S_4bb65bc_60011761_E, "%h" , ID_S_7c95915f_1ba7b2ab_E);                if( &ID_S_b886fae_7e68b8e4_E == 1'b1)        begin         $display("FATAL: UNKNOWN FILE FORMAT.");         $stop;        end         ID_S_6d11d081_2ba6b70c_E = ~ID_S_6d11d081_2ba6b70c_E;        ->ID_S_793b3647_16adf866_E;       end      else       begin        $display("FATAL: UNKNOWN FILE FORMAT. EXPECTING: COMMAND or DATA");        $stop;       end           end         end endmodule 