//////////////////////////////////////////////////////////////////
//                                                             
// Copyright (C) 2010 Pulse Logic                       
//                    info@pulselogic.com.pl                   
//                                                             
// This source file may be used and distributed without        
// restriction provided that this copyright statement is not   
// removed from the file and that any derivative work contains 
// the original copyright notice and the associated disclaimer.
//                                                             
//     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
// POSSIBILITY OF SUCH DAMAGE.                                 
//                                                             
//////////////////////////////////////////////////////////////////
//
// Title       : AHB definitions
// Design      : AHB Testbench
// Company     : Pulse Logic
//
//////////////////////////////////////////////////////////////////
//
// File        : defs.v
// File Type   : RTL (Obfuscated code)
// Tool        : Xilinx WebPack
//
//////////////////////////////////////////////////////////////////
//
// Description : AHB constants
//////////////////////////////////////////////////////////////////

//AHB constants.
localparam C_IDLE = 2'b00;
localparam C_NONSEQ = 2'b10;
localparam C_SEQ =  2'b11;
localparam C_BUSY = 2'b01;
//Burst type.
localparam C_SINGLE = 3'b000;
localparam C_INCR = 3'b001;
localparam C_WRAP4 = 3'b010;
localparam C_INCR4 = 3'b011;
localparam C_WRAP8 = 3'b100;
localparam C_INCR8 = 3'b101;
localparam C_WRAP16 = 3'b110;
localparam C_INCR16 = 3'b111;
//Transfer size.
localparam C_BITS8 = 3'b000; 
localparam C_BITS16 = 3'b001; 
localparam C_BITS32 = 3'b010;
localparam C_BITS64 = 3'b011; 
localparam C_BITS128 = 3'b100; 
localparam C_BITS256 = 3'b101; 
localparam C_BITS512 = 3'b110; 
localparam C_BITS1024 = 3'b111; 
//Salve response.
localparam C_OKAY = 2'b00;
localparam C_ERROR = 2'b01; 
localparam C_RETRY = 2'b10;
localparam C_SPLIT = 2'b11;

//Transfer direction.
localparam C_READ = 1'b0;
localparam C_WRITE = 1'b1;	

//Address of dummy slave.
localparam C_DUMMY_SLV_ADDR = 0;

//Calculate number of beats for a burts.
function integer AhbNumOfCycles (input [2 : 0] burst);		
	integer res;
begin	
	case (burst)
		C_SINGLE , C_INCR	:	res = 1;
		C_WRAP4  , C_INCR4	:   res = 4;
		C_WRAP8  , C_INCR8	:   res = 8;
		C_WRAP16 , C_INCR16	: 	res = 16;
	endcase
	AhbNumOfCycles = res;
end
endfunction 

//log2 function to calculate range of some ports.
function integer clog2 (input integer param);		
	integer log2;
begin		
	for(log2 = 0; param > 0; log2 = log2 + 1)
	begin
		param = param >> 1;
	end
	clog2 = log2;
end
endfunction
