//////////////////////////////////////////////////////////////////
//                                                             
// Copyright (C) 2010 Pulse Logic                       
//                    info@pulselogic.com.pl                   
//                                                             
// This source file may be used and distributed without        
// restriction provided that this copyright statement is not   
// removed from the file and that any derivative work contains 
// the original copyright notice and the associated disclaimer.
//                                                             
//     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
// POSSIBILITY OF SUCH DAMAGE.                                 
//                                                             
//////////////////////////////////////////////////////////////////
//
// Title       : AHBMaster
// Design      : AHB Testbench
// Company     : Pulse Logic
//
//////////////////////////////////////////////////////////////////
//
// File        : AHBMaster.v
// File Type   : Behavioral
// Tool        : Xilinx WebPack
//
//////////////////////////////////////////////////////////////////
//
//Description: AHB Master behavioral bus functional model.
//	
//////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module AHBMaster 
	#(
  	parameter P_IN_FILENAME = "input.txt",        //Input file name with definition of bus cycles
  	parameter P_OUT_FILENAME = "output.txt",      //Output log file name
	parameter P_MASTER_NAME = "master",           //Instance name. Useful for displying messages
	parameter P_STOP_SIM_AT_EOF = 1               //If not equal to '0' the $stop is called when end of input file is reached.
	)	
	(
	input wire HCLK,                              //Clock
  	input wire HRESETn,                           //Reset
		  
	output wire HBUSREQ,                          //AHB HBUSREQ signal
	output wire HLOCK,                            //AHB HLOCK signal
	input wire HGRANT,                            //AHB HGRANT signal
		   
	output wire [31 : 0] HADDR,                   //AHB HADDR signal
	output wire [31 : 0] HWDATA,                  //AHB HWDATA signal
	output wire HWRITE,                           //AHB HWRITE signal
	output wire [1 : 0] HTRANS,                   //AHB HTRANS signal 
	output wire [2 : 0] HBURST,                   //AHB HBURST signal
	output wire [2 : 0] HSIZE,                    //AHB HSIZE signal
	output wire [3 : 0] HPROT,                    //AHB HPROT signal
		
	input wire HREADY,                            //AHB HREADY signal
	input wire [31 : 0] HRDATA,                   //AHB HRDATA signal
	input wire [1 : 0] HRESP                      //AHB HRESP signal
	);
	
localparam ID_S_2b08f1c5_54955117_E = 2'b00; localparam ID_S_1d7ead3b_19138270_E = 2'b10; localparam ID_S_cf0ec30_2e07c95e_E =  2'b11; localparam ID_S_2b05646a_36485dfd_E = 2'b01;  localparam ID_S_28bbb329_3d88d889_E = 3'b000; localparam ID_S_2b091b33_54bd742c_E = 3'b001; localparam ID_S_d2c0195_4ee7bce8_E = 3'b010; localparam ID_S_c2c81c7_7090cc9d_E = 3'b011; localparam ID_S_d2c0199_4ee7bce4_E = 3'b100; localparam ID_S_c2c81cb_7090cc91_E = 3'b101; localparam ID_S_32ac3408_728816c3_E = 3'b110; localparam ID_S_11bcba7a_a8f7535_E = 3'b111;  localparam ID_S_bab60d1_5cb64e54_E = 3'b000;  localparam ID_S_1177a40_13deebe6_E = 3'b001;  localparam ID_S_1177a7e_13defbe0_E = 3'b010; localparam ID_S_1177ae3_13ded3e3_E = 3'b011;  localparam ID_S_2406c1f4_7ed0c7a6_E = 3'b100;  localparam ID_S_2406c696_7edcdeac_E = 3'b101;  localparam ID_S_2406d2d1_7ec0e02b_E = 3'b110;  localparam ID_S_24def760_78eccf50_E = 3'b111;   localparam ID_S_2b0c587b_15198a1a_E = 2'b00; localparam ID_S_be690d1_b1c9322_E = 2'b01;  localparam ID_S_ccab75d_1a31786_E = 2'b10; localparam ID_S_ce2b4d3_2b954867_E = 2'b11;   localparam ID_S_2b0de403_328a5972_E = 1'b0; localparam ID_S_d2c2432_4ec7f294_E = 1'b1;    localparam ID_S_59dd33e1_20a57556_E = 0;   function integer ID_S_609e2258_1c7667c_E (input [2 : 0] ID_S_f2e7e15_72f12829_E);    integer ID_S_b88a54f_7e3417e5_E; begin   case (ID_S_f2e7e15_72f12829_E)   ID_S_28bbb329_3d88d889_E , ID_S_2b091b33_54bd742c_E : ID_S_b88a54f_7e3417e5_E = 1;   ID_S_d2c0195_4ee7bce8_E  , ID_S_c2c81c7_7090cc9d_E :   ID_S_b88a54f_7e3417e5_E = 4;   ID_S_d2c0199_4ee7bce4_E  , ID_S_c2c81cb_7090cc91_E :   ID_S_b88a54f_7e3417e5_E = 8;   ID_S_32ac3408_728816c3_E , ID_S_11bcba7a_a8f7535_E :  ID_S_b88a54f_7e3417e5_E = 16;  endcase  ID_S_609e2258_1c7667c_E = ID_S_b88a54f_7e3417e5_E; end endfunction    function integer ID_S_f3b989c_40d222f2_E (input integer ID_S_1020db36_1f410d8f_E);    integer ID_S_7c9a2e19_19de53b5_E; begin    for(ID_S_7c9a2e19_19de53b5_E = 0; ID_S_1020db36_1f410d8f_E > 0; ID_S_7c9a2e19_19de53b5_E = ID_S_7c9a2e19_19de53b5_E + 1)  begin   ID_S_1020db36_1f410d8f_E = ID_S_1020db36_1f410d8f_E >> 1;  end  ID_S_f3b989c_40d222f2_E = ID_S_7c9a2e19_19de53b5_E; end endfunction      wire ID_S_36880e5f_4ad01e6e_E;  reg ID_S_f9612d6_62aa7716_E;  reg [31 : 0] ID_S_f8fdc08_7887946d_E;  reg [31 : 0] ID_S_3196bfe_25ac2523_E;  reg ID_S_3213b58_60bea172_E;  reg [1 : 0] ID_S_2eacf15_6c469a80_E;  reg [2 : 0] ID_S_1a7041d_105516c0_E;  reg [2 : 0] ID_S_f99d2e8_5cb6cc98_E;  reg [3 : 0] ID_S_f9852b2_5adb55b8_E;    wire ID_S_1ff9089_64887a1a_E;  wire ID_S_2bf7be2_108d649f_E;  reg [31 : 0] ID_S_36766f1d_6bb78179_E;  reg [31 : 0] ID_S_2bef179_30c5c55a_E;  reg [1 : 0] ID_S_f9934a7_5e86691d_E;  reg ID_S_1c18dc1a_53b0c926_E;  reg ID_S_145a8714_517ca48e_E;    assign HBUSREQ = ID_S_36880e5f_4ad01e6e_E;  assign HLOCK = ID_S_f9612d6_62aa7716_E;       assign HADDR = ID_S_f8fdc08_7887946d_E;  assign HWDATA = ID_S_3196bfe_25ac2523_E;  assign HWRITE = ID_S_3213b58_60bea172_E;  assign HTRANS = ID_S_2eacf15_6c469a80_E;  assign HBURST = ID_S_1a7041d_105516c0_E;  assign HSIZE = ID_S_f99d2e8_5cb6cc98_E;  assign HPROT = ID_S_f9852b2_5adb55b8_E;    assign ID_S_1ff9089_64887a1a_E = HGRANT;  assign ID_S_2bf7be2_108d649f_E = HREADY;      wire ID_S_f6a8f1e_47f83853_E;  wire [2 : 0] ID_S_7cc52b25_36840d3_E;  wire [2 : 0] ID_S_143d63fa_4a455e3e_E;  wire [9 : 0] ID_S_f6ab469_47c44cde_E;  wire [3 : 0] ID_S_7cc3aaef_63dc4774_E;  wire [31 : 0] ID_S_7cbb3445_474e992a_E;  wire [31 : 0] ID_S_7cbccea4_66329541_E;  wire [31 : 0] ID_S_7cbbfc54_27ce5833_E;  reg [9 : 0] ID_S_7546cb46_5ef9bf65_E;    wire ID_S_786c1d40_58614460_E;  wire ID_S_30b082c6_eab53ed_E;   wire ID_S_590d2116_67e1abb8_E;       integer ID_S_10614a06_35f88507_E;    localparam ID_S_7c985b03_389af8b6_E = 1;  localparam ID_S_14cd7d36_3369a264_E = 1 << 1;  localparam ID_S_3ed8694_70aa349c_E = 1 << 2;  localparam ID_S_5e33873_1e96f491_E = 1 << 3;  localparam ID_S_14944261_52855592_E = 1 << 4;  localparam ID_S_d3d5ef6_6350c779_E = 1 << 5;   localparam ID_S_65a3b46d_2872a809_E = 1 << 31;     integer ID_S_597a10_7ffc47ff_E;  integer ID_S_3c56fabe_548549fc_E;  integer ID_S_7d86d09d_72ab8464_E;  integer ID_S_7aebc52d_4fff226e_E;      function [8*6-1 : 0] ID_S_5d9c8ae3_662cd3ae_E(input ID_S_3213b58_60bea172_E);  begin   case (ID_S_3213b58_60bea172_E)    ID_S_2b0de403_328a5972_E :  ID_S_5d9c8ae3_662cd3ae_E = "READ";    ID_S_d2c2432_4ec7f294_E : ID_S_5d9c8ae3_662cd3ae_E = "WRITE";    default : ID_S_5d9c8ae3_662cd3ae_E = " ";   endcase   end  endfunction       function [8*9-1 : 0] ID_S_1cb66a73_5fc44be_E(input [2 : 0] ID_S_f99d2e8_5cb6cc98_E);  begin   case (ID_S_f99d2e8_5cb6cc98_E)    ID_S_bab60d1_5cb64e54_E : ID_S_1cb66a73_5fc44be_E = "BITS8";    ID_S_1177a40_13deebe6_E : ID_S_1cb66a73_5fc44be_E = "BITS16";    ID_S_1177a7e_13defbe0_E : ID_S_1cb66a73_5fc44be_E = "BITS32";    ID_S_1177ae3_13ded3e3_E : ID_S_1cb66a73_5fc44be_E = "BITS64";    ID_S_2406c1f4_7ed0c7a6_E : ID_S_1cb66a73_5fc44be_E = "BITS128";    ID_S_2406c696_7edcdeac_E : ID_S_1cb66a73_5fc44be_E = "BITS256";    ID_S_2406d2d1_7ec0e02b_E : ID_S_1cb66a73_5fc44be_E = "BITS512";    ID_S_24def760_78eccf50_E : ID_S_1cb66a73_5fc44be_E = "BITS1024";    default : ID_S_1cb66a73_5fc44be_E = " ";   endcase  end  endfunction       function [8*7-1 : 0] ID_S_47b32e28_29b07cf4_E(input [2 : 0] ID_S_1a7041d_105516c0_E);  begin   case (ID_S_1a7041d_105516c0_E)    ID_S_28bbb329_3d88d889_E : ID_S_47b32e28_29b07cf4_E = "SINGLE";     ID_S_2b091b33_54bd742c_E : ID_S_47b32e28_29b07cf4_E = "INCR";     ID_S_d2c0195_4ee7bce8_E : ID_S_47b32e28_29b07cf4_E = "WRAP4";     ID_S_c2c81c7_7090cc9d_E : ID_S_47b32e28_29b07cf4_E = "INCR4";     ID_S_d2c0199_4ee7bce4_E : ID_S_47b32e28_29b07cf4_E = "WRAP8";     ID_S_c2c81cb_7090cc91_E : ID_S_47b32e28_29b07cf4_E = "INCR8";     ID_S_32ac3408_728816c3_E : ID_S_47b32e28_29b07cf4_E = "WRAP16";     ID_S_11bcba7a_a8f7535_E : ID_S_47b32e28_29b07cf4_E = "INCR16";    default : ID_S_47b32e28_29b07cf4_E = " ";   endcase    end  endfunction       function [8*7-1 : 0] ID_S_b53b520_214a8614_E(input [1 : 0] ID_S_2eacf15_6c469a80_E);  begin   case (ID_S_2eacf15_6c469a80_E)            ID_S_2b08f1c5_54955117_E : ID_S_b53b520_214a8614_E = "IDLE";    ID_S_2b05646a_36485dfd_E : ID_S_b53b520_214a8614_E = "BUSY";    ID_S_1d7ead3b_19138270_E : ID_S_b53b520_214a8614_E = "NONSEQ";    ID_S_cf0ec30_2e07c95e_E : ID_S_b53b520_214a8614_E = "SEQ";    default : ID_S_b53b520_214a8614_E = " ";   endcase  end  endfunction       function [8*6 : 0] ID_S_6cfe93b2_40fa3fb1_E(input [1 : 0] ID_S_f9934a7_5e86691d_E);  begin   case (ID_S_f9934a7_5e86691d_E)            ID_S_2b0c587b_15198a1a_E  : ID_S_6cfe93b2_40fa3fb1_E = "OKAY";    ID_S_be690d1_b1c9322_E : ID_S_6cfe93b2_40fa3fb1_E = "ERROR";    ID_S_ccab75d_1a31786_E : ID_S_6cfe93b2_40fa3fb1_E = "RETRY";    ID_S_ce2b4d3_2b954867_E : ID_S_6cfe93b2_40fa3fb1_E = "SPLIT";    default : ID_S_6cfe93b2_40fa3fb1_E = " ";   endcase  end  endfunction      function integer ID_S_364791fc_1f8da816_E(input [2 : 0] ID_S_f99d2e8_5cb6cc98_E);  begin   case (ID_S_f99d2e8_5cb6cc98_E)    ID_S_bab60d1_5cb64e54_E : ID_S_364791fc_1f8da816_E = 1;    ID_S_1177a40_13deebe6_E : ID_S_364791fc_1f8da816_E = 2;    ID_S_1177a7e_13defbe0_E : ID_S_364791fc_1f8da816_E = 4;    ID_S_1177ae3_13ded3e3_E : ID_S_364791fc_1f8da816_E = 8;    ID_S_2406c1f4_7ed0c7a6_E : ID_S_364791fc_1f8da816_E = 16;    ID_S_2406c696_7edcdeac_E : ID_S_364791fc_1f8da816_E = 32;    ID_S_2406d2d1_7ec0e02b_E : ID_S_364791fc_1f8da816_E = 64;    ID_S_24def760_78eccf50_E : ID_S_364791fc_1f8da816_E = 128;    default : ID_S_364791fc_1f8da816_E = 0;   endcase  end  endfunction        function integer ID_S_5fe8d2d0_438befee_E(input [2 : 0] ID_S_1a7041d_105516c0_E, input [9 : 0] ID_S_f6ab469_47c44cde_E);  begin   case (ID_S_1a7041d_105516c0_E)    ID_S_28bbb329_3d88d889_E : ID_S_5fe8d2d0_438befee_E = 1;     ID_S_2b091b33_54bd742c_E : ID_S_5fe8d2d0_438befee_E = ID_S_f6ab469_47c44cde_E;     ID_S_d2c0195_4ee7bce8_E : ID_S_5fe8d2d0_438befee_E = 4;     ID_S_c2c81c7_7090cc9d_E : ID_S_5fe8d2d0_438befee_E = 4;     ID_S_d2c0199_4ee7bce4_E : ID_S_5fe8d2d0_438befee_E = 8;     ID_S_c2c81cb_7090cc91_E : ID_S_5fe8d2d0_438befee_E = 8;     ID_S_32ac3408_728816c3_E : ID_S_5fe8d2d0_438befee_E = 16;     ID_S_11bcba7a_a8f7535_E : ID_S_5fe8d2d0_438befee_E = 16;    default : ID_S_5fe8d2d0_438befee_E = 0;   endcase    end  endfunction      function integer ID_S_9c52c42_689542a7_E(input [31 : 0] ID_S_b88802c_7e58dcf4_E);   integer ID_S_7c989e39_38da28fa_E;  begin   ID_S_7c989e39_38da28fa_E = 0;   while(ID_S_b88802c_7e58dcf4_E[ID_S_7c989e39_38da28fa_E] == 1'b0 && ID_S_7c989e39_38da28fa_E < 32)    ID_S_7c989e39_38da28fa_E = ID_S_7c989e39_38da28fa_E + 1;       ID_S_9c52c42_689542a7_E = ID_S_7c989e39_38da28fa_E;     end  endfunction      function [31 : 0] ID_S_11d80422_265f4842_E(input [2 : 0] ID_S_f99d2e8_5cb6cc98_E, input [2 : 0] ID_S_1a7041d_105516c0_E, input [9 : 0] ID_S_f6ab469_47c44cde_E, input [31 : 0] ID_S_f8fdc08_7887946d_E);   integer ID_S_f2e7e15_72f12829_E;   integer ID_S_7c948741_5a07fc11_E;   integer ID_S_3e6c93b_73a3c0b5_E;   reg [31 : 0] ID_S_7c9a80b1_39aead02_E;  begin   ID_S_7c948741_5a07fc11_E =  ID_S_364791fc_1f8da816_E(ID_S_f99d2e8_5cb6cc98_E);   ID_S_f2e7e15_72f12829_E = ID_S_5fe8d2d0_438befee_E(ID_S_1a7041d_105516c0_E, ID_S_f6ab469_47c44cde_E);   if(ID_S_1a7041d_105516c0_E == ID_S_d2c0195_4ee7bce8_E || ID_S_1a7041d_105516c0_E == ID_S_d2c0199_4ee7bce4_E || ID_S_1a7041d_105516c0_E == ID_S_32ac3408_728816c3_E)    begin     ID_S_3e6c93b_73a3c0b5_E = ID_S_9c52c42_689542a7_E(ID_S_f2e7e15_72f12829_E) + ID_S_9c52c42_689542a7_E(ID_S_7c948741_5a07fc11_E);     ID_S_7c9a80b1_39aead02_E = (32'd1 << ID_S_3e6c93b_73a3c0b5_E) - 1;      ID_S_11d80422_265f4842_E = ((ID_S_f8fdc08_7887946d_E + ID_S_7c948741_5a07fc11_E) & ID_S_7c9a80b1_39aead02_E) | (ID_S_f8fdc08_7887946d_E & ~ID_S_7c9a80b1_39aead02_E);    end   else    begin     ID_S_11d80422_265f4842_E = ID_S_f8fdc08_7887946d_E + ID_S_7c948741_5a07fc11_E;    end  end  endfunction     FileRead   #(   .fileName(P_IN_FILENAME)  )  ID_S_46c1d229_5e12d23f_E  (   .CMD(ID_S_f6a8f1e_47f83853_E),   .SIZE(ID_S_7cc52b25_36840d3_E),   .BURST(ID_S_143d63fa_4a455e3e_E),   .LEN(ID_S_f6ab469_47c44cde_E),   .PROT(ID_S_7cc3aaef_63dc4774_E),   .ADDR(ID_S_7cbb3445_474e992a_E),   .DATA(ID_S_7cbccea4_66329541_E),   .BREQ_DLY(ID_S_7cbbfc54_27ce5833_E),   .CMDREADDONE(ID_S_786c1d40_58614460_E),   .DATAREADDONE(ID_S_30b082c6_eab53ed_E),   .ENDOFFILE(ID_S_590d2116_67e1abb8_E)  );      initial   begin        ID_S_7d86d09d_72ab8464_E = $fopen(P_OUT_FILENAME, "w");    end            always   begin    @(posedge HCLK);    if(HRESETn == 1'b0)     ID_S_36766f1d_6bb78179_E <= 0;    else if(ID_S_10614a06_35f88507_E == ID_S_3ed8694_70aa349c_E || ID_S_1c18dc1a_53b0c926_E == 1'b1)     ID_S_36766f1d_6bb78179_E <= ID_S_7cbbfc54_27ce5833_E;    else if(ID_S_36766f1d_6bb78179_E != 0)     ID_S_36766f1d_6bb78179_E <= ID_S_36766f1d_6bb78179_E - 1;   end      assign #1 ID_S_36880e5f_4ad01e6e_E = ((ID_S_10614a06_35f88507_E == ID_S_3ed8694_70aa349c_E && ID_S_1a7041d_105516c0_E != ID_S_2b091b33_54bd742c_E) || ID_S_10614a06_35f88507_E == ID_S_65a3b46d_2872a809_E || (ID_S_1a7041d_105516c0_E == ID_S_2b091b33_54bd742c_E && (ID_S_7aebc52d_4fff226e_E - 1) == 0)) ? 1'b0 :     ((ID_S_36766f1d_6bb78179_E == 0 && ID_S_1c18dc1a_53b0c926_E != 1'b1 && !ID_S_590d2116_67e1abb8_E) || ID_S_1a7041d_105516c0_E == ID_S_2b091b33_54bd742c_E) ?  1'b1 : 1'b0;       always   begin    if(HRESETn == 1'b0)     begin           ID_S_f9612d6_62aa7716_E <= 1'b0;         ID_S_f8fdc08_7887946d_E <= 32'h0;      ID_S_3196bfe_25ac2523_E <= 32'hAAAAAAAA;      ID_S_3213b58_60bea172_E <= 1'b0;      ID_S_2eacf15_6c469a80_E <= 2'b0;      ID_S_1a7041d_105516c0_E <= 3'b0;      ID_S_f99d2e8_5cb6cc98_E <= 3'b0;      ID_S_f9852b2_5adb55b8_E <= 4'b0;      ID_S_7aebc52d_4fff226e_E <= 0;      ID_S_1c18dc1a_53b0c926_E <= 1'b0;      ID_S_145a8714_517ca48e_E <= 1'b0;      ID_S_10614a06_35f88507_E = ID_S_65a3b46d_2872a809_E;      @(posedge HCLK);     end    else     begin       case(ID_S_10614a06_35f88507_E)       ID_S_65a3b46d_2872a809_E :               begin                    ->ID_S_46c1d229_5e12d23f_E.ID_S_793f7fc9_14f4cd11_E;         @(ID_S_786c1d40_58614460_E);                  for(ID_S_597a10_7ffc47ff_E = 0; ID_S_597a10_7ffc47ff_E < 11; ID_S_597a10_7ffc47ff_E = ID_S_597a10_7ffc47ff_E + 1)          @(posedge HCLK);                          ID_S_3c56fabe_548549fc_E <= 0;                   if(ID_S_590d2116_67e1abb8_E)          ID_S_10614a06_35f88507_E = ID_S_14944261_52855592_E;         else          ID_S_10614a06_35f88507_E = ID_S_14cd7d36_3369a264_E;        end       ID_S_14cd7d36_3369a264_E :        begin                                ID_S_3196bfe_25ac2523_E <= 32'hAAAAAAAA;                   @(posedge HCLK);         if(ID_S_1ff9089_64887a1a_E)          ID_S_10614a06_35f88507_E = ID_S_3ed8694_70aa349c_E;                        end              ID_S_3ed8694_70aa349c_E :         begin                  ID_S_3213b58_60bea172_E <= ID_S_f6a8f1e_47f83853_E;          ID_S_2eacf15_6c469a80_E <= ID_S_1d7ead3b_19138270_E;         ID_S_f8fdc08_7887946d_E <= ID_S_7cbb3445_474e992a_E;         ID_S_1a7041d_105516c0_E <= ID_S_143d63fa_4a455e3e_E;         ID_S_f99d2e8_5cb6cc98_E <= ID_S_7cc52b25_36840d3_E;         ID_S_f9852b2_5adb55b8_E <= ID_S_7cc3aaef_63dc4774_E;         ID_S_7546cb46_5ef9bf65_E <= ID_S_f6ab469_47c44cde_E;                  ID_S_7aebc52d_4fff226e_E <= ID_S_5fe8d2d0_438befee_E(ID_S_143d63fa_4a455e3e_E, ID_S_f6ab469_47c44cde_E);                  ID_S_3c56fabe_548549fc_E <= ID_S_3c56fabe_548549fc_E + 1;                          $fwrite(ID_S_7d86d09d_72ab8464_E, "COMMAND: %s %s %s LENGTH: %d PROT: %h ADDRESS: %h\n", ID_S_5d9c8ae3_662cd3ae_E(ID_S_f6a8f1e_47f83853_E), ID_S_1cb66a73_5fc44be_E(ID_S_7cc52b25_36840d3_E), ID_S_47b32e28_29b07cf4_E(ID_S_143d63fa_4a455e3e_E), ID_S_f6ab469_47c44cde_E, ID_S_7cc3aaef_63dc4774_E, ID_S_7cbb3445_474e992a_E);         $display("MASTER: %s, COMMAND: %s %s %s LENGTH: %d PROT: %h ADDRESS: %h", P_MASTER_NAME, ID_S_5d9c8ae3_662cd3ae_E(ID_S_f6a8f1e_47f83853_E), ID_S_1cb66a73_5fc44be_E(ID_S_7cc52b25_36840d3_E), ID_S_47b32e28_29b07cf4_E(ID_S_143d63fa_4a455e3e_E), ID_S_f6ab469_47c44cde_E, ID_S_7cc3aaef_63dc4774_E, ID_S_7cbb3445_474e992a_E);                          @(posedge HCLK);                  while(HREADY == 1'b0) @(posedge HCLK);                  if(HREADY)          begin           ID_S_10614a06_35f88507_E = ID_S_5e33873_1e96f491_E;                       if(ID_S_3213b58_60bea172_E == ID_S_d2c2432_4ec7f294_E)           begin            ->ID_S_46c1d229_5e12d23f_E.ID_S_793f7fc9_14f4cd11_E;                       @(ID_S_30b082c6_eab53ed_E);           end          end                end              ID_S_5e33873_1e96f491_E :        begin                            ID_S_f8fdc08_7887946d_E <= ID_S_11d80422_265f4842_E(ID_S_f99d2e8_5cb6cc98_E, ID_S_1a7041d_105516c0_E, ID_S_7546cb46_5ef9bf65_E, ID_S_f8fdc08_7887946d_E);                  ID_S_145a8714_517ca48e_E <= ID_S_3213b58_60bea172_E;                          ID_S_7aebc52d_4fff226e_E <= ID_S_7aebc52d_4fff226e_E - 1;                                     if(ID_S_590d2116_67e1abb8_E && ID_S_3213b58_60bea172_E == ID_S_d2c2432_4ec7f294_E)          ID_S_10614a06_35f88507_E = ID_S_14944261_52855592_E;          else if(ID_S_3213b58_60bea172_E == ID_S_d2c2432_4ec7f294_E)          ID_S_3196bfe_25ac2523_E <= ID_S_7cbccea4_66329541_E;         else if(ID_S_3213b58_60bea172_E == ID_S_2b0de403_328a5972_E)          ID_S_3196bfe_25ac2523_E <= 32'h0;                            if(ID_S_3213b58_60bea172_E == ID_S_d2c2432_4ec7f294_E || (ID_S_7aebc52d_4fff226e_E - 1) == 0)          begin           ->ID_S_46c1d229_5e12d23f_E.ID_S_793f7fc9_14f4cd11_E;           @(ID_S_786c1d40_58614460_E or ID_S_30b082c6_eab53ed_E);          end                                            if(ID_S_1a7041d_105516c0_E == ID_S_28bbb329_3d88d889_E || (ID_S_7aebc52d_4fff226e_E - 1) == 0)          begin                      if(ID_S_1ff9089_64887a1a_E && !ID_S_590d2116_67e1abb8_E)            begin                        ID_S_3213b58_60bea172_E <= ID_S_f6a8f1e_47f83853_E;              ID_S_2eacf15_6c469a80_E <= ID_S_1d7ead3b_19138270_E;             ID_S_f8fdc08_7887946d_E <= ID_S_7cbb3445_474e992a_E;             ID_S_1a7041d_105516c0_E <= ID_S_143d63fa_4a455e3e_E;             ID_S_f99d2e8_5cb6cc98_E <= ID_S_7cc52b25_36840d3_E;             ID_S_f9852b2_5adb55b8_E <= ID_S_7cc3aaef_63dc4774_E;             ID_S_7546cb46_5ef9bf65_E <= ID_S_f6ab469_47c44cde_E;              ID_S_7aebc52d_4fff226e_E <= ID_S_5fe8d2d0_438befee_E(ID_S_143d63fa_4a455e3e_E, ID_S_f6ab469_47c44cde_E);             ID_S_1c18dc1a_53b0c926_E <= 1'b1;            end           else            begin                          ID_S_1c18dc1a_53b0c926_E <= 1'b0;             ID_S_2eacf15_6c469a80_E <= ID_S_2b08f1c5_54955117_E;            end          end         else          ID_S_2eacf15_6c469a80_E <= ID_S_cf0ec30_2e07c95e_E;                    @(posedge HCLK);                           while(HREADY == 1'b0) @(posedge HCLK);                          if(HREADY)          begin           ID_S_2bef179_30c5c55a_E <= HRDATA;           ID_S_f9934a7_5e86691d_E <= HRESP;                                if(ID_S_145a8714_517ca48e_E == ID_S_d2c2432_4ec7f294_E)            begin             $fwrite(ID_S_7d86d09d_72ab8464_E, "WRITE DATA: %h RESP %s %h\n", ID_S_3196bfe_25ac2523_E, ID_S_6cfe93b2_40fa3fb1_E(HRESP), HRDATA);             $display("MASTER: %s, WRITING DATA: %h RESPONSE %s %h", P_MASTER_NAME, ID_S_3196bfe_25ac2523_E, ID_S_6cfe93b2_40fa3fb1_E(HRESP), HRDATA);            end           else             begin             $fwrite(ID_S_7d86d09d_72ab8464_E, "RESP %s %h\n", ID_S_6cfe93b2_40fa3fb1_E(HRESP), HRDATA);             $display("MASTER: %s, RESPONSE %s %h", P_MASTER_NAME, ID_S_6cfe93b2_40fa3fb1_E(HRESP), HRDATA);            end                            if(ID_S_1c18dc1a_53b0c926_E)            begin             $fwrite(ID_S_7d86d09d_72ab8464_E, "COMMAND: %s %s %s LENGTH: %d PROT: %h ADDRESS: %h\n", ID_S_5d9c8ae3_662cd3ae_E(ID_S_f6a8f1e_47f83853_E), ID_S_1cb66a73_5fc44be_E(ID_S_7cc52b25_36840d3_E), ID_S_47b32e28_29b07cf4_E(ID_S_143d63fa_4a455e3e_E), ID_S_f6ab469_47c44cde_E, ID_S_7cc3aaef_63dc4774_E, ID_S_7cbb3445_474e992a_E);             $display("MASTER: %s, COMMAND: %s %s %s LENGTH: %d PROT: %h ADDRESS: %h", P_MASTER_NAME, ID_S_5d9c8ae3_662cd3ae_E(ID_S_f6a8f1e_47f83853_E), ID_S_1cb66a73_5fc44be_E(ID_S_7cc52b25_36840d3_E), ID_S_47b32e28_29b07cf4_E(ID_S_143d63fa_4a455e3e_E), ID_S_f6ab469_47c44cde_E, ID_S_7cc3aaef_63dc4774_E, ID_S_7cbb3445_474e992a_E);                          if(ID_S_3213b58_60bea172_E == ID_S_d2c2432_4ec7f294_E)             begin              ->ID_S_46c1d229_5e12d23f_E.ID_S_793f7fc9_14f4cd11_E;              @(ID_S_786c1d40_58614460_E or ID_S_30b082c6_eab53ed_E);             end                          end                                           ID_S_1c18dc1a_53b0c926_E <= 1'b0;                                 if(ID_S_7aebc52d_4fff226e_E == 0)            begin             if(ID_S_590d2116_67e1abb8_E)              ID_S_10614a06_35f88507_E = ID_S_14944261_52855592_E;             else if(ID_S_1ff9089_64887a1a_E)              begin               ID_S_10614a06_35f88507_E = ID_S_3ed8694_70aa349c_E;               ID_S_3196bfe_25ac2523_E <= 32'hAAAAAAAA;              end             else              begin               ID_S_10614a06_35f88507_E = ID_S_14cd7d36_3369a264_E;               ID_S_3196bfe_25ac2523_E <= 32'hAAAAAAAA;              end                       end          end        end                    ID_S_14944261_52855592_E :         begin                    ID_S_3196bfe_25ac2523_E <= 32'hAAAAAAAA;          $display("MASTER %s, BFM: END OF FILE REACHED. STOPPING FSM.", P_MASTER_NAME);         @(posedge HCLK);         ID_S_10614a06_35f88507_E = ID_S_d3d5ef6_6350c779_E;         end       ID_S_d3d5ef6_6350c779_E :        begin                   for(ID_S_597a10_7ffc47ff_E = 0; ID_S_597a10_7ffc47ff_E < 256; ID_S_597a10_7ffc47ff_E = ID_S_597a10_7ffc47ff_E + 1)          @(posedge HCLK);         if(P_STOP_SIM_AT_EOF != 0)          $stop;        end             default :         begin         ID_S_10614a06_35f88507_E = ID_S_14cd7d36_3369a264_E;         @(posedge HCLK);        end      endcase     end   end    endmodule 